@@ -18,9 +18,9 @@ enum lcCommandId
 	LC_FILE_EXPORT_POVRAY,
 	LC_FILE_EXPORT_WAVEFRONT,
 	LC_FILE_RENDER,
+	LC_FILE_INSTRUCTIONS,
 	LC_FILE_PRINT,
 	LC_FILE_PRINT_PREVIEW,
-	LC_FILE_PRINT_BOM,
 	LC_FILE_RECENT_FIRST,
 	LC_FILE_RECENT1 = LC_FILE_RECENT_FIRST,
 	LC_FILE_RECENT2,
@@ -33,14 +33,18 @@ enum lcCommandId
 	LC_EDIT_CUT,
 	LC_EDIT_COPY,
 	LC_EDIT_PASTE,
+	LC_EDIT_PASTE_STEPS,
 	LC_EDIT_FIND,
 	LC_EDIT_FIND_NEXT,
 	LC_EDIT_FIND_PREVIOUS,
+	LC_EDIT_FIND_ALL,
+	LC_EDIT_REPLACE,
+	LC_EDIT_REPLACE_NEXT,
+	LC_EDIT_REPLACE_ALL,
 	LC_EDIT_SELECT_ALL,
 	LC_EDIT_SELECT_NONE,
 	LC_EDIT_SELECT_INVERT,
 	LC_EDIT_SELECT_BY_NAME,
-	LC_EDIT_SELECT_BY_COLOR,
 	LC_EDIT_SELECTION_MODE_FIRST,
 	LC_EDIT_SELECTION_SINGLE = LC_EDIT_SELECTION_MODE_FIRST,
 	LC_EDIT_SELECTION_PIECE,
@@ -48,6 +52,11 @@ enum lcCommandId
 	LC_EDIT_SELECTION_PIECE_COLOR,
 	LC_EDIT_SELECTION_MODE_LAST = LC_EDIT_SELECTION_PIECE_COLOR,
 	LC_EDIT_TRANSFORM_RELATIVE,
+	LC_EDIT_TRANSFORM_ABSOLUTE,
+	LC_EDIT_TRANSFORM_TOGGLE_RELATIVE,
+	LC_EDIT_TRANSFORM_SEPARATELY,
+	LC_EDIT_TRANSFORM_TOGETHER,
+	LC_EDIT_TRANSFORM_TOGGLE_SEPARATE,
 	LC_EDIT_SNAP_MOVE_TOGGLE,
 	LC_EDIT_SNAP_MOVE_XY0,
 	LC_EDIT_SNAP_MOVE_XY1,
@@ -95,6 +104,7 @@ enum lcCommandId
 	LC_EDIT_ACTION_ROTATE,
 	LC_EDIT_ACTION_DELETE,
 	LC_EDIT_ACTION_PAINT,
+	LC_EDIT_ACTION_COLOR_PICKER,
 	LC_EDIT_ACTION_ZOOM,
 	LC_EDIT_ACTION_PAN,
 	LC_EDIT_ACTION_ROTATE_VIEW,
@@ -139,18 +149,28 @@ enum lcCommandId
 	LC_VIEW_CAMERA15,
 	LC_VIEW_CAMERA16,
 	LC_VIEW_CAMERA_LAST = LC_VIEW_CAMERA16,
-	LC_VIEW_CAMERA_RESET,
 	LC_VIEW_TIME_FIRST,
 	LC_VIEW_TIME_PREVIOUS,
 	LC_VIEW_TIME_NEXT,
 	LC_VIEW_TIME_LAST,
-	LC_VIEW_TIME_INSERT,
+	LC_VIEW_TIME_INSERT_BEFORE,
+	LC_VIEW_TIME_INSERT_AFTER,
 	LC_VIEW_TIME_DELETE,
 	LC_VIEW_TIME_ADD_KEYS,
 	LC_VIEW_SPLIT_HORIZONTAL,
 	LC_VIEW_SPLIT_VERTICAL,
 	LC_VIEW_REMOVE_VIEW,
 	LC_VIEW_RESET_VIEWS,
+	LC_VIEW_TOOLBAR_STANDARD,
+	LC_VIEW_TOOLBAR_FIRST = LC_VIEW_TOOLBAR_STANDARD,
+	LC_VIEW_TOOLBAR_TOOLS,
+	LC_VIEW_TOOLBAR_TIME,
+	LC_VIEW_TOOLBAR_PARTS,
+	LC_VIEW_TOOLBAR_COLORS,
+	LC_VIEW_TOOLBAR_PROPERTIES,
+	LC_VIEW_TOOLBAR_TIMELINE,
+	LC_VIEW_TOOLBAR_PREVIEW,
+	LC_VIEW_TOOLBAR_LAST = LC_VIEW_TOOLBAR_PREVIEW,
 	LC_VIEW_FULLSCREEN,
 	LC_VIEW_CLOSE_CURRENT_TAB,
 	LC_VIEW_SHADING_FIRST,
@@ -162,9 +182,14 @@ enum lcCommandId
 	LC_VIEW_PROJECTION_PERSPECTIVE = LC_VIEW_PROJECTION_FIRST,
 	LC_VIEW_PROJECTION_ORTHO,
 	LC_VIEW_PROJECTION_LAST = LC_VIEW_PROJECTION_ORTHO,
+	LC_VIEW_TOGGLE_VIEW_SPHERE,
+	LC_VIEW_TOGGLE_AXIS_ICON,
+	LC_VIEW_TOGGLE_GRID,
+	LC_VIEW_FADE_PREVIOUS_STEPS,
 	LC_PIECE_INSERT,
 	LC_PIECE_DELETE,
 	LC_PIECE_DUPLICATE,
+	LC_PIECE_PAINT_SELECTED,
 	LC_PIECE_RESET_PIVOT_POINT,
 	LC_PIECE_REMOVE_KEY_FRAMES,
 	LC_PIECE_CONTROL_POINT_INSERT,
@@ -227,12 +252,29 @@ enum lcCommandId
 	LC_MODEL_22,
 	LC_MODEL_23,
 	LC_MODEL_24,
-	LC_MODEL_LAST = LC_MODEL_24,
+	LC_MODEL_25,
+	LC_MODEL_26,
+	LC_MODEL_27,
+	LC_MODEL_28,
+	LC_MODEL_29,
+	LC_MODEL_30,
+	LC_MODEL_31,
+	LC_MODEL_32,
+	LC_MODEL_33,
+	LC_MODEL_34,
+	LC_MODEL_35,
+	LC_MODEL_36,
+	LC_MODEL_37,
+	LC_MODEL_38,
+	LC_MODEL_39,
+	LC_MODEL_40,
+	LC_MODEL_LAST = LC_MODEL_40,
 	LC_HELP_HOMEPAGE,
 	LC_HELP_BUG_REPORT,
 	LC_HELP_UPDATES,
 	LC_HELP_ABOUT,
-	LC_TIMELINE_INSERT,
+	LC_TIMELINE_INSERT_BEFORE,
+	LC_TIMELINE_INSERT_AFTER,
 	LC_TIMELINE_DELETE,
 	LC_TIMELINE_MOVE_SELECTION,
 	LC_TIMELINE_SET_CURRENT,
@@ -247,26 +289,27 @@ struct lcCommand
 	const char* DefaultShortcut;
 };
 
-extern lcCommand gCommands[LC_NUM_COMMANDS];
+extern const lcCommand gCommands[];
 
-enum lcTool
+enum class lcTool
 {
-	LC_TOOL_INSERT,
-	LC_TOOL_LIGHT,
-	LC_TOOL_SPOTLIGHT,
-	LC_TOOL_CAMERA,
-	LC_TOOL_SELECT,
-	LC_TOOL_MOVE,
-	LC_TOOL_ROTATE,
-	LC_TOOL_ERASER,
-	LC_TOOL_PAINT,
-	LC_TOOL_ZOOM,
-	LC_TOOL_PAN,
-	LC_TOOL_ROTATE_VIEW,
-	LC_TOOL_ROLL,
-	LC_TOOL_ZOOM_REGION,
-	LC_NUM_TOOLS
+	Insert,
+	Light,
+	SpotLight,
+	Camera,
+	Select,
+	Move,
+	Rotate,
+	Eraser,
+	Paint,
+	ColorPicker,
+	Zoom,
+	Pan,
+	RotateView,
+	Roll,
+	ZoomRegion,
+	Count
 };
 
-extern const char* gToolNames[LC_NUM_TOOLS];
+extern const char* gToolNames[];
 
