@@ -6,6 +6,12 @@
 #include "lc_colors.h"
 #include "lc_mainwindow.h"
 #include "lc_library.h"
+#include "texfont.h"
+#include "lc_view.h"
+#include "lc_viewsphere.h"
+#include "lc_stringcache.h"
+#include "lc_partselectionwidget.h"
+#include <QOpenGLFunctions_3_2_Core>
 
 #ifdef LC_OPENGLES
 #define glEnableClientState(...)
@@ -18,7 +24,10 @@
 #define GL_STATIC_DRAW_ARB GL_STATIC_DRAW
 #endif
 
-lcProgram lcContext::mPrograms[LC_NUM_MATERIALS];
+std::unique_ptr<QOpenGLContext> lcContext::mOffscreenContext;
+std::unique_ptr<QOffscreenSurface> lcContext::mOffscreenSurface;
+std::unique_ptr<lcContext> lcContext::mGlobalOffscreenContext;
+lcProgram lcContext::mPrograms[static_cast<int>(lcMaterialType::Count)];
 
 lcContext::lcContext()
 {
@@ -33,17 +42,17 @@ lcContext::lcContext()
 	mColorEnabled = false;
 
 	mTexture2D = 0;
-	mTexture2DMS = 0;
 	mTextureCubeMap = 0;
-	mPolygonOffset = LC_POLYGON_OFFSET_NONE;
+	mPolygonOffset = lcPolygonOffset::None;
+	mDepthWrite = true;
+	mDepthFunction = lcDepthFunction::LessEqual;
+	mCullFace = false;
 	mLineWidth = 1.0f;
 #ifndef LC_OPENGLES
 	mMatrixMode = GL_MODELVIEW;
 	mTextureEnabled = false;
 #endif
 
-	mFramebufferObject = 0;
-
 	mColor = lcVector4(0.0f, 0.0f, 0.0f, 0.0f);
 	mWorldMatrix = lcMatrix44Identity();
 	mViewMatrix = lcMatrix44Identity();
@@ -60,13 +69,107 @@ lcContext::lcContext()
 	mViewProjectionMatrixDirty = false;
 	mHighlightParamsDirty = false;
 
-	mMaterialType = LC_NUM_MATERIALS;
+	mMaterialType = lcMaterialType::Count;
 }
 
 lcContext::~lcContext()
 {
 }
 
+bool lcContext::InitializeRenderer()
+{
+	if (!CreateOffscreenContext())
+		return false;
+
+	mGlobalOffscreenContext = std::unique_ptr<lcContext>(new(lcContext));
+
+	lcContext* Context = mGlobalOffscreenContext.get();
+	Context->SetOffscreenContext();
+
+	lcInitializeGLExtensions(mOffscreenContext.get());
+
+	// TODO: Find a better place for the grid texture and font
+	gStringCache.Initialize(Context);
+	gTexFont.Initialize(Context);
+
+	Context->CreateResources();
+	lcView::CreateResources(Context);
+	lcViewSphere::CreateResources(Context);
+
+	if (!gSupportsShaderObjects && lcGetPreferences().mShadingMode == lcShadingMode::DefaultLights)
+		lcGetPreferences().mShadingMode = lcShadingMode::Flat;
+
+	if (!gSupportsShaderObjects && lcGetPreferences().mDrawConditionalLines)
+		lcGetPreferences().mDrawConditionalLines = false;
+
+	if (!gSupportsFramebufferObject)
+		gMainWindow->GetPartSelectionWidget()->DisableIconMode();
+
+	return true;
+}
+
+void lcContext::ShutdownRenderer()
+{
+	if (!mGlobalOffscreenContext)
+		return;
+
+	mGlobalOffscreenContext->MakeCurrent();
+	lcContext* Context = mGlobalOffscreenContext.get();
+
+	gStringCache.Reset();
+	gTexFont.Reset();
+
+	lcView::DestroyResources(Context);
+	Context->DestroyResources();
+	lcViewSphere::DestroyResources(Context);
+
+	mGlobalOffscreenContext.reset();
+
+	lcContext::DestroyOffscreenContext();
+}
+
+lcContext* lcContext::GetGlobalOffscreenContext()
+{
+	return mGlobalOffscreenContext.get();
+}
+
+bool lcContext::CreateOffscreenContext()
+{
+	std::unique_ptr<QOpenGLContext> OffscreenContext(new QOpenGLContext());
+
+	if (!OffscreenContext)
+		return false;
+
+	OffscreenContext->setShareContext(QOpenGLContext::globalShareContext());
+
+	if (!OffscreenContext->create() || !OffscreenContext->isValid())
+		return false;
+
+	std::unique_ptr<QOffscreenSurface> OffscreenSurface(new QOffscreenSurface());
+
+	if (!OffscreenSurface)
+		return false;
+
+	OffscreenSurface->create();
+
+	if (!OffscreenSurface->isValid())
+		return false;
+
+	if (!OffscreenContext->makeCurrent(OffscreenSurface.get()))
+		return false;
+
+	mOffscreenContext = std::move(OffscreenContext);
+	mOffscreenSurface = std::move(OffscreenSurface);
+
+	return true;
+}
+
+void lcContext::DestroyOffscreenContext()
+{
+	mOffscreenSurface.reset();
+	mOffscreenContext.reset();
+}
+
 void lcContext::CreateShaderPrograms()
 {
 	const char* ShaderPrefix =
@@ -101,46 +204,45 @@ void lcContext::CreateShaderPrograms()
 "		LC_SHADER_PRECISION float Diffuse = min(abs(dot(Normal, LightDirection)) * 0.6 + 0.65, 1.0);\n"
 	};
 
-	const char* VertexShaders[LC_NUM_MATERIALS] =
+	const char* const VertexShaders[] =
 	{
-		":/resources/shaders/unlit_color_vs.glsl",            // LC_MATERIAL_UNLIT_COLOR
-		":/resources/shaders/unlit_texture_modulate_vs.glsl", // LC_MATERIAL_UNLIT_TEXTURE_MODULATE
-		":/resources/shaders/unlit_texture_decal_vs.glsl",    // LC_MATERIAL_UNLIT_TEXTURE_DECAL
-		":/resources/shaders/unlit_vertex_color_vs.glsl",     // LC_MATERIAL_UNLIT_VERTEX_COLOR
-		":/resources/shaders/unlit_view_sphere_vs.glsl",      // LC_MATERIAL_UNLIT_VIEW_SPHERE
-		":/resources/shaders/fakelit_color_vs.glsl",          // LC_MATERIAL_FAKELIT_COLOR
-		":/resources/shaders/fakelit_texture_decal_vs.glsl"   // LC_MATERIAL_FAKELIT_TEXTURE_DECAL
+		":/resources/shaders/unlit_color_vs.glsl",             // UnlitColor
+		":/resources/shaders/unlit_color_conditional_vs.glsl", // UnlitColorConditional
+		":/resources/shaders/unlit_texture_modulate_vs.glsl",  // UnlitTextureModulate
+		":/resources/shaders/unlit_texture_decal_vs.glsl",     // UnlitTextureDecal
+		":/resources/shaders/unlit_vertex_color_vs.glsl",      // UnlitVertexColor
+		":/resources/shaders/unlit_view_sphere_vs.glsl",       // UnlitViewSphere
+		":/resources/shaders/fakelit_color_vs.glsl",           // FakeLitColor
+		":/resources/shaders/fakelit_texture_decal_vs.glsl"    // FakeLitTextureDecal
 	};
 
-	const char* FragmentShaders[LC_NUM_MATERIALS] =
+	LC_ARRAY_SIZE_CHECK(VertexShaders, lcMaterialType::Count);
+
+	const char* const FragmentShaders[] =
 	{
-		":/resources/shaders/unlit_color_ps.glsl",            // LC_MATERIAL_UNLIT_COLOR
-		":/resources/shaders/unlit_texture_modulate_ps.glsl", // LC_MATERIAL_UNLIT_TEXTURE_MODULATE
-		":/resources/shaders/unlit_texture_decal_ps.glsl",    // LC_MATERIAL_UNLIT_TEXTURE_DECAL
-		":/resources/shaders/unlit_vertex_color_ps.glsl",     // LC_MATERIAL_UNLIT_VERTEX_COLOR
-		":/resources/shaders/unlit_view_sphere_ps.glsl",      // LC_MATERIAL_UNLIT_VIEW_SPHERE
-		":/resources/shaders/fakelit_color_ps.glsl",          // LC_MATERIAL_FAKELIT_COLOR
-		":/resources/shaders/fakelit_texture_decal_ps.glsl"   // LC_MATERIAL_FAKELIT_TEXTURE_DECAL
+		":/resources/shaders/unlit_color_ps.glsl",             // UnlitColor
+		":/resources/shaders/unlit_color_conditional_ps.glsl", // UnlitColorConditional
+		":/resources/shaders/unlit_texture_modulate_ps.glsl",  // UnlitTextureModulate
+		":/resources/shaders/unlit_texture_decal_ps.glsl",     // UnlitTextureDecal
+		":/resources/shaders/unlit_vertex_color_ps.glsl",      // UnlitVertexColor
+		":/resources/shaders/unlit_view_sphere_ps.glsl",       // UnlitViewSphere
+		":/resources/shaders/fakelit_color_ps.glsl",           // FakeLitColor
+		":/resources/shaders/fakelit_texture_decal_ps.glsl"    // FakeLitTextureDecal
 	};
 
-	auto LoadShader = [ShaderPrefix](const char* FileName, GLuint ShaderType) -> GLuint
-	{
-		QResource Resource(FileName);
+	LC_ARRAY_SIZE_CHECK(FragmentShaders, lcMaterialType::Count);
 
-		if (!Resource.isValid())
+	const auto LoadShader = [this, ShaderPrefix](const char* FileName, GLuint ShaderType) -> GLuint
+	{
+		QFile ShaderFile(FileName);
+
+		if (!ShaderFile.open(QIODevice::ReadOnly))
 			return 0;
 
-		QByteArray Data;
-
-		if (Resource.isCompressed())
-			Data = qUncompress(Resource.data(), Resource.size());
-		else
-			Data = QByteArray::fromRawData((const char*)Resource.data(), Resource.size());
-
-		Data = ShaderPrefix + Data;
+		QByteArray Data = ShaderPrefix + ShaderFile.readAll();
 		const char* Source = Data.constData();
 
-		GLuint Shader = glCreateShader(ShaderType);
+		const GLuint Shader = glCreateShader(ShaderType);
 		glShaderSource(Shader, 1, &Source, nullptr);
 		glCompileShader(Shader);
 
@@ -164,20 +266,25 @@ void lcContext::CreateShaderPrograms()
 		return Shader;
 	};
 
-	for (int MaterialType = 0; MaterialType < LC_NUM_MATERIALS; MaterialType++)
+	for (int MaterialType = 0; MaterialType < static_cast<int>(lcMaterialType::Count); MaterialType++)
 	{
-		GLuint VertexShader = LoadShader(VertexShaders[MaterialType], GL_VERTEX_SHADER);
-		GLuint FragmentShader = LoadShader(FragmentShaders[MaterialType], GL_FRAGMENT_SHADER);
+		const GLuint VertexShader = LoadShader(VertexShaders[MaterialType], GL_VERTEX_SHADER);
+		const GLuint FragmentShader = LoadShader(FragmentShaders[MaterialType], GL_FRAGMENT_SHADER);
 
 		GLuint Program = glCreateProgram();
 
 		glAttachShader(Program, VertexShader);
 		glAttachShader(Program, FragmentShader);
 
-		glBindAttribLocation(Program, LC_ATTRIB_POSITION, "VertexPosition");
-		glBindAttribLocation(Program, LC_ATTRIB_NORMAL, "VertexNormal");
-		glBindAttribLocation(Program, LC_ATTRIB_TEXCOORD, "VertexTexCoord");
-		glBindAttribLocation(Program, LC_ATTRIB_COLOR, "VertexColor");
+		glBindAttribLocation(Program, static_cast<int>(lcProgramAttrib::Position), "VertexPosition");
+		glBindAttribLocation(Program, static_cast<int>(lcProgramAttrib::Normal), "VertexNormal");
+		glBindAttribLocation(Program, static_cast<int>(lcProgramAttrib::TexCoord), "VertexTexCoord");
+		glBindAttribLocation(Program, static_cast<int>(lcProgramAttrib::Color), "VertexColor");
+
+		glBindAttribLocation(Program, static_cast<int>(lcProgramAttrib::ControlPoint1), "VertexPosition1");
+		glBindAttribLocation(Program, static_cast<int>(lcProgramAttrib::ControlPoint2), "VertexPosition2");
+		glBindAttribLocation(Program, static_cast<int>(lcProgramAttrib::ControlPoint3), "VertexPosition3");
+		glBindAttribLocation(Program, static_cast<int>(lcProgramAttrib::ControlPoint4), "VertexPosition4");
 
 		glLinkProgram(Program);
 
@@ -209,6 +316,15 @@ void lcContext::CreateShaderPrograms()
 		mPrograms[MaterialType].LightPositionLocation = glGetUniformLocation(Program, "LightPosition");
 		mPrograms[MaterialType].EyePositionLocation = glGetUniformLocation(Program, "EyePosition");
 		mPrograms[MaterialType].HighlightParamsLocation = glGetUniformLocation(Program, "HighlightParams");
+
+		GLint TextureLocation = glGetUniformLocation(Program, "Texture");
+
+		if (TextureLocation != -1)
+		{
+			glUseProgram(Program);
+			glUniform1i(TextureLocation, 0);
+			glUseProgram(0);
+		}
 	}
 }
 
@@ -225,17 +341,45 @@ void lcContext::DestroyResources()
 	if (!gSupportsShaderObjects)
 		return;
 
-	for (int MaterialType = 0; MaterialType < LC_NUM_MATERIALS; MaterialType++)
+	for (int MaterialType = 0; MaterialType < static_cast<int>(lcMaterialType::Count); MaterialType++)
 	{
 		glDeleteProgram(mPrograms[MaterialType].Object);
 		mPrograms[MaterialType].Object = 0;
 	}
 }
 
+void lcContext::MakeCurrent()
+{
+	if (mWidget)
+		mWidget->makeCurrent();
+	else
+		mOffscreenContext->makeCurrent(mOffscreenSurface.get());
+}
+
+void lcContext::SetGLContext(QOpenGLContext* Context, QOpenGLWidget* Widget)
+{
+	mContext = Context;
+	mWidget = Widget;
+
+	MakeCurrent();
+	initializeOpenGLFunctions();
+}
+
+void lcContext::SetOffscreenContext()
+{
+	SetGLContext(mOffscreenContext.get(), nullptr);
+}
+
 void lcContext::SetDefaultState()
 {
+#ifndef LC_OPENGLES
+	if (QSurfaceFormat::defaultFormat().samples() > 1)
+		glEnable(GL_LINE_SMOOTH);
+#endif
+
 	glEnable(GL_DEPTH_TEST);
 	glDepthFunc(GL_LEQUAL);
+	mDepthFunction = lcDepthFunction::LessEqual;
 
 	if (gSupportsBlendFuncSeparate)
 		glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE_MINUS_DST_ALPHA, GL_ONE);
@@ -250,10 +394,15 @@ void lcContext::SetDefaultState()
 
 	if (gSupportsShaderObjects)
 	{
-		glEnableVertexAttribArray(LC_ATTRIB_POSITION);
-		glDisableVertexAttribArray(LC_ATTRIB_NORMAL);
-		glDisableVertexAttribArray(LC_ATTRIB_TEXCOORD);
-		glDisableVertexAttribArray(LC_ATTRIB_COLOR);
+		SetVertexAttribPointer(lcProgramAttrib::Position, 3, GL_FLOAT, false, 0, nullptr);
+		EnableVertexAttrib(lcProgramAttrib::Position);
+
+		DisableVertexAttrib(lcProgramAttrib::Normal);
+		SetVertexAttribPointer(lcProgramAttrib::Normal, 4, GL_BYTE, true, 0, nullptr);
+		DisableVertexAttrib(lcProgramAttrib::TexCoord);
+		SetVertexAttribPointer(lcProgramAttrib::TexCoord, 2, GL_FLOAT, false, 0, nullptr);
+		DisableVertexAttrib(lcProgramAttrib::Color);
+		SetVertexAttribPointer(lcProgramAttrib::Color, 4, GL_FLOAT, false, 0, nullptr);
 	}
 	else
 	{
@@ -267,33 +416,35 @@ void lcContext::SetDefaultState()
 		glNormalPointer(GL_BYTE, 0, nullptr);
 		glTexCoordPointer(2, GL_FLOAT, 0, nullptr);
 		glColorPointer(4, GL_FLOAT, 0, nullptr);
+
+		mNormalEnabled = false;
+		mTexCoordEnabled = false;
+		mColorEnabled = false;
 #endif
 	}
 
-	mNormalEnabled = false;
-	mTexCoordEnabled = false;
-	mColorEnabled = false;
-
 	mVertexBufferObject = 0;
 	mIndexBufferObject = 0;
 	mVertexBufferPointer = nullptr;
 	mIndexBufferPointer = nullptr;
 	mVertexBufferOffset = (char*)~0;
 
+	for (int AttribIndex = 0; AttribIndex < static_cast<int>(lcProgramAttrib::Count); AttribIndex++)
+		mVertexAttribState[AttribIndex] = lcVertexAttribState();
+
 	glBindTexture(GL_TEXTURE_2D, 0);
 	mTexture2D = 0;
-#ifndef LC_OPENGLES
-	if (gSupportsTexImage2DMultisample)
-	{
-		glBindTexture(GL_TEXTURE_2D_MULTISAMPLE, 0);
-		mTexture2DMS = 0;
-	}
-#endif
 	glBindTexture(GL_TEXTURE_CUBE_MAP, 0);
 	mTextureCubeMap = 0;
 
 	glDisable(GL_POLYGON_OFFSET_FILL);
-	mPolygonOffset = LC_POLYGON_OFFSET_NONE;
+	mPolygonOffset = lcPolygonOffset::None;
+
+	mDepthWrite = true;
+	glDepthMask(GL_TRUE);
+
+	glDisable(GL_CULL_FACE);
+	mCullFace = false;
 
 	glLineWidth(1.0f);
 	mLineWidth = 1.0f;
@@ -301,7 +452,7 @@ void lcContext::SetDefaultState()
 	if (gSupportsShaderObjects)
 	{
 		glUseProgram(0);
-		mMaterialType = LC_NUM_MATERIALS;
+		mMaterialType = lcMaterialType::Count;
 	}
 	else
 	{
@@ -316,12 +467,22 @@ void lcContext::SetDefaultState()
 	}
 }
 
+void lcContext::ClearColorAndDepth(const lcVector4& ClearColor)
+{
+	glClearColor(ClearColor[0], ClearColor[1], ClearColor[2], ClearColor[3]);
+	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+}
+
+void lcContext::ClearDepth()
+{
+	glClear(GL_DEPTH_BUFFER_BIT);
+}
+
 void lcContext::ClearResources()
 {
 	ClearVertexBuffer();
 	ClearIndexBuffer();
 	BindTexture2D(0);
-	BindTexture2DMS(0);
 }
 
 void lcContext::SetMaterial(lcMaterialType MaterialType)
@@ -333,7 +494,7 @@ void lcContext::SetMaterial(lcMaterialType MaterialType)
 
 	if (gSupportsShaderObjects)
 	{
-		glUseProgram(mPrograms[MaterialType].Object);
+		glUseProgram(mPrograms[static_cast<int>(MaterialType)].Object);
 		mColorDirty = true;
 		mWorldMatrixDirty = true; // todo: change dirty to a bitfield and set the lighting constants dirty here
 		mViewMatrixDirty = true;
@@ -344,7 +505,7 @@ void lcContext::SetMaterial(lcMaterialType MaterialType)
 #ifndef LC_OPENGLES
 		switch (MaterialType)
 		{
-		case LC_MATERIAL_UNLIT_TEXTURE_MODULATE:
+		case lcMaterialType::UnlitTextureModulate:
 			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
 			if (!mTextureEnabled)
@@ -354,8 +515,8 @@ void lcContext::SetMaterial(lcMaterialType MaterialType)
 			}
 			break;
 
-		case LC_MATERIAL_FAKELIT_TEXTURE_DECAL:
-		case LC_MATERIAL_UNLIT_TEXTURE_DECAL:
+		case lcMaterialType::FakeLitTextureDecal:
+		case lcMaterialType::UnlitTextureDecal:
 			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
 
 			if (!mTextureEnabled)
@@ -365,9 +526,10 @@ void lcContext::SetMaterial(lcMaterialType MaterialType)
 			}
 			break;
 
-		case LC_MATERIAL_UNLIT_COLOR:
-		case LC_MATERIAL_UNLIT_VERTEX_COLOR:
-		case LC_MATERIAL_FAKELIT_COLOR:
+		case lcMaterialType::UnlitColor:
+		case lcMaterialType::UnlitColorConditional:
+		case lcMaterialType::UnlitVertexColor:
+		case lcMaterialType::FakeLitColor:
 			if (mTextureEnabled)
 			{
 				glDisable(GL_TEXTURE_2D);
@@ -375,8 +537,8 @@ void lcContext::SetMaterial(lcMaterialType MaterialType)
 			}
 			break;
 
-		case LC_MATERIAL_UNLIT_VIEW_SPHERE:
-		case LC_NUM_MATERIALS:
+		case lcMaterialType::UnlitViewSphere:
+		case lcMaterialType::Count:
 			break;
 		}
 #endif
@@ -395,16 +557,16 @@ void lcContext::SetPolygonOffset(lcPolygonOffset PolygonOffset)
 
 	switch (PolygonOffset)
 	{
-	case LC_POLYGON_OFFSET_NONE:
+	case lcPolygonOffset::None:
 		glDisable(GL_POLYGON_OFFSET_FILL);
 		break;
 
-	case LC_POLYGON_OFFSET_OPAQUE:
+	case lcPolygonOffset::Opaque:
 		glPolygonOffset(0.5f, 0.1f);
 		glEnable(GL_POLYGON_OFFSET_FILL);
 		break;
 
-	case LC_POLYGON_OFFSET_TRANSLUCENT:
+	case lcPolygonOffset::Translucent:
 		glPolygonOffset(0.25f, 0.1f);
 		glEnable(GL_POLYGON_OFFSET_FILL);
 		break;
@@ -413,6 +575,47 @@ void lcContext::SetPolygonOffset(lcPolygonOffset PolygonOffset)
 	mPolygonOffset = PolygonOffset;
 }
 
+void lcContext::SetDepthWrite(bool Enable)
+{
+	if (Enable == mDepthWrite)
+		return;
+
+	glDepthMask(Enable ? GL_TRUE : GL_FALSE);
+	mDepthWrite = Enable;
+}
+
+void lcContext::SetDepthFunction(lcDepthFunction DepthFunction)
+{
+	if (DepthFunction == mDepthFunction)
+		return;
+
+	switch (DepthFunction)
+	{
+	case lcDepthFunction::Always:
+		glDepthFunc(GL_ALWAYS);
+		break;
+
+	case lcDepthFunction::LessEqual:
+		glDepthFunc(GL_LEQUAL);
+		break;
+	}
+
+	mDepthFunction = DepthFunction;
+}
+
+void lcContext::EnableCullFace(bool Enable)
+{
+	if (Enable == mCullFace)
+		return;
+
+	if (Enable)
+		glEnable(GL_CULL_FACE);
+	else
+		glDisable(GL_CULL_FACE);
+
+	mCullFace = Enable;
+}
+
 void lcContext::SetLineWidth(float LineWidth)
 {
 	if (LineWidth == mLineWidth)
@@ -430,20 +633,6 @@ void lcContext::SetSmoothShading(bool Smooth)
 #endif
 }
 
-void lcContext::BeginTranslucent()
-{
-	glEnable(GL_BLEND);
-	glDepthMask(GL_FALSE);
-	SetPolygonOffset(LC_POLYGON_OFFSET_TRANSLUCENT);
-}
-
-void lcContext::EndTranslucent()
-{
-	glDepthMask(GL_TRUE);
-	glDisable(GL_BLEND);
-	SetPolygonOffset(LC_POLYGON_OFFSET_OPAQUE);
-}
-
 void lcContext::BindTexture2D(GLuint Texture)
 {
 	if (mTexture2D == Texture)
@@ -453,20 +642,6 @@ void lcContext::BindTexture2D(GLuint Texture)
 	mTexture2D = Texture;
 }
 
-void lcContext::BindTexture2DMS(GLuint Texture)
-{
-	if (mTexture2DMS == Texture)
-		return;
-
-#ifndef LC_OPENGLES
-	if (gSupportsTexImage2DMultisample)
-	{
-		glBindTexture(GL_TEXTURE_2D_MULTISAMPLE, Texture);
-		mTexture2DMS = Texture;
-	}
-#endif
-}
-
 void lcContext::BindTextureCubeMap(GLuint Texture)
 {
 	if (mTextureCubeMap == Texture)
@@ -506,224 +681,30 @@ void lcContext::SetColorIndex(int ColorIndex)
 
 void lcContext::SetColorIndexTinted(int ColorIndex, lcInterfaceColor InterfaceColor, float Weight)
 {
-	lcVector3 Color(gColorList[ColorIndex].Value * Weight + gInterfaceColors[InterfaceColor] * (1.0f - Weight));
+	const lcVector3 Color(gColorList[ColorIndex].Value * Weight + gInterfaceColors[InterfaceColor] * (1.0f - Weight));
 	SetColor(lcVector4(Color, gColorList[ColorIndex].Value.w));
 }
 
+void lcContext::SetColorIndexTinted(int ColorIndex, const lcVector4& Tint)
+{
+	SetColor(gColorList[ColorIndex].Value * Tint);
+}
+
 void lcContext::SetEdgeColorIndex(int ColorIndex)
 {
 	SetColor(gColorList[ColorIndex].Edge);
 }
 
+void lcContext::SetEdgeColorIndexTinted(int ColorIndex, const lcVector4& Tint)
+{
+	SetColor(gColorList[ColorIndex].Edge * Tint);
+}
+
 void lcContext::SetInterfaceColor(lcInterfaceColor InterfaceColor)
 {
 	SetColor(gInterfaceColors[InterfaceColor]);
 }
 
-void lcContext::ClearFramebuffer()
-{
-	if (!mFramebufferObject)
-		return;
-
-	if (gSupportsFramebufferObjectARB)
-		glBindFramebuffer(GL_FRAMEBUFFER, 0);
-#ifndef LC_OPENGLES
-	else if (gSupportsFramebufferObjectEXT)
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
-#endif
-
-	mFramebufferObject = 0;
-}
-
-lcFramebuffer lcContext::CreateFramebuffer(int Width, int Height, bool Depth, bool Multisample)
-{
-	lcFramebuffer Framebuffer(Width, Height);
-
-	if (gSupportsFramebufferObjectARB)
-	{
-		int Samples = (Multisample && gSupportsTexImage2DMultisample && QGLFormat::defaultFormat().sampleBuffers()) ? QGLFormat::defaultFormat().samples() : 1;
-
-		glGenFramebuffers(1, &Framebuffer.mObject);
-		glBindFramebuffer(GL_FRAMEBUFFER, Framebuffer.mObject);
-
-		glGenTextures(1, &Framebuffer.mColorTexture);
-		if (Depth)
-			glGenRenderbuffers(1, &Framebuffer.mDepthRenderbuffer);
-
-		if (Samples == 1)
-		{
-			BindTexture2D(Framebuffer.mColorTexture);
-			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
-			BindTexture2D(0);
-			glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, Framebuffer.mColorTexture, 0);
-
-			if (Depth)
-			{
-				glBindRenderbuffer(GL_RENDERBUFFER, Framebuffer.mDepthRenderbuffer);
-				glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT24, Width, Height);
-				glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, Framebuffer.mDepthRenderbuffer);
-			}
-		}
-#ifndef LC_OPENGLES
-		else
-		{
-			BindTexture2DMS(Framebuffer.mColorTexture);
-			glTexImage2DMultisample(GL_TEXTURE_2D_MULTISAMPLE, Samples, GL_RGBA, Width, Height, GL_TRUE);
-			BindTexture2DMS(0);
-			glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D_MULTISAMPLE, Framebuffer.mColorTexture, 0);
-
-			if (Depth)
-			{
-				glBindRenderbuffer(GL_RENDERBUFFER, Framebuffer.mDepthRenderbuffer);
-				glRenderbufferStorageMultisample(GL_RENDERBUFFER, Samples, GL_DEPTH_COMPONENT24, Width, Height);
-				glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, Framebuffer.mDepthRenderbuffer);
-			}
-		}
-#endif
-		
-		if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
-			DestroyFramebuffer(Framebuffer);
-
-		glBindFramebuffer(GL_FRAMEBUFFER, mFramebufferObject);
-	}
-#ifndef LC_OPENGLES
-	else if (gSupportsFramebufferObjectEXT)
-	{
-		glGenFramebuffersEXT(1, &Framebuffer.mObject);
-		glGenTextures(1, &Framebuffer.mColorTexture);
-
-		BindTexture2D(Framebuffer.mColorTexture);
-		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
-
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, Framebuffer.mObject);
-		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, Framebuffer.mColorTexture, 0);
-
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, Framebuffer.mObject);
-
-		glGenRenderbuffersEXT(1, &Framebuffer.mDepthRenderbuffer);
-		glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, Framebuffer.mDepthRenderbuffer);
-		glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, Width, Height);
-
-		glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, Framebuffer.mDepthRenderbuffer);
-
-		BindTexture2D(0);
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, Framebuffer.mObject);
-
-		if (glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT) != GL_FRAMEBUFFER_COMPLETE_EXT)
-			DestroyFramebuffer(Framebuffer);
-
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFramebufferObject);
-	}
-#endif
-
-	return Framebuffer;
-}
-
-void lcContext::DestroyFramebuffer(lcFramebuffer& Framebuffer)
-{
-	if (gSupportsFramebufferObjectARB)
-	{
-		glDeleteFramebuffers(1, &Framebuffer.mObject);
-		glDeleteTextures(1, &Framebuffer.mColorTexture);
-		glDeleteRenderbuffers(1, &Framebuffer.mDepthRenderbuffer);
-	}
-#ifndef LC_OPENGLES
-	else if (gSupportsFramebufferObjectEXT)
-	{
-		glDeleteFramebuffersEXT(1, &Framebuffer.mObject);
-		glDeleteTextures(1, &Framebuffer.mColorTexture);
-		glDeleteRenderbuffersEXT(1, &Framebuffer.mDepthRenderbuffer);
-	}
-#endif
-
-	Framebuffer.mObject = 0;
-	Framebuffer.mColorTexture = 0;
-	Framebuffer.mDepthRenderbuffer = 0;
-	Framebuffer.mWidth = 0;
-	Framebuffer.mHeight = 0;
-}
-
-void lcContext::BindFramebuffer(GLuint FramebufferObject)
-{
-	if (FramebufferObject == mFramebufferObject)
-		return;
-
-	if (gSupportsFramebufferObjectARB)
-		glBindFramebuffer(GL_FRAMEBUFFER, FramebufferObject);
-#ifndef LC_OPENGLES
-	else if (gSupportsFramebufferObjectEXT)
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, FramebufferObject);
-#endif
-
-	mFramebufferObject = FramebufferObject;
-}
-
-std::pair<lcFramebuffer, lcFramebuffer> lcContext::CreateRenderFramebuffer(int Width, int Height)
-{
-	if (gSupportsFramebufferObjectARB && QGLFormat::defaultFormat().sampleBuffers() && QGLFormat::defaultFormat().samples() > 1)
-		return std::make_pair(CreateFramebuffer(Width, Height, true, true), CreateFramebuffer(Width, Height, false, false));
-	else
-		return std::make_pair(CreateFramebuffer(Width, Height, true, false), lcFramebuffer());
-}
-
-void lcContext::DestroyRenderFramebuffer(std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer)
-{
-	DestroyFramebuffer(RenderFramebuffer.first);
-	DestroyFramebuffer(RenderFramebuffer.second);
-}
-
-QImage lcContext::GetRenderFramebufferImage(const std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer)
-{
-	QImage Image(RenderFramebuffer.first.mWidth, RenderFramebuffer.first.mHeight, QImage::Format_ARGB32);
-
-	GetRenderFramebufferImage(RenderFramebuffer, Image.bits());
-
-	return Image;
-}
-
-void lcContext::GetRenderFramebufferImage(const std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer, quint8* Buffer)
-{
-	const int Width = RenderFramebuffer.first.mWidth;
-	const int Height = RenderFramebuffer.first.mHeight;
-	GLuint SavedFramebuffer = mFramebufferObject;
-
-	if (RenderFramebuffer.second.IsValid())
-	{
-#ifndef LC_OPENGLES
-		glBindFramebuffer(GL_DRAW_FRAMEBUFFER, RenderFramebuffer.second.mObject);
-		glBindFramebuffer(GL_READ_FRAMEBUFFER, RenderFramebuffer.first.mObject);
-		glBlitFramebuffer(0, 0, Width, Height, 0, 0, Width, Height, GL_COLOR_BUFFER_BIT, GL_LINEAR);
-		BindFramebuffer(RenderFramebuffer.second);
-#endif
-	}
-	else
-		BindFramebuffer(RenderFramebuffer.first);
-
-	glFinish();
-	glReadPixels(0, 0, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE, Buffer);
-	BindFramebuffer(SavedFramebuffer);
-
-	for (int y = 0; y < (Height + 1) / 2; y++)
-	{
-		quint8* Top = Buffer + ((Height - y - 1) * Width * 4);
-		quint8* Bottom = Buffer + y * Width * 4;
-
-		for (int x = 0; x < Width; x++)
-		{
-			QRgb TopColor = qRgba(Top[0], Top[1], Top[2], Top[3]);
-			QRgb BottomColor = qRgba(Bottom[0], Bottom[1], Bottom[2], Bottom[3]);
-
-			*(QRgb*)Top = BottomColor;
-			*(QRgb*)Bottom = TopColor;
-
-			Top += 4;
-			Bottom += 4;
-		}
-	}
-}
-
 lcVertexBuffer lcContext::CreateVertexBuffer(int Size, const void* Data)
 {
 	lcVertexBuffer VertexBuffer;
@@ -829,19 +810,14 @@ void lcContext::ClearVertexBuffer()
 
 	if (gSupportsShaderObjects)
 	{
-		if (mNormalEnabled)
-			glDisableVertexAttribArray(LC_ATTRIB_NORMAL);
+		SetVertexAttribPointer(lcProgramAttrib::Position, 3, GL_FLOAT, false, 0, nullptr);
 
-		if (mTexCoordEnabled)
-			glDisableVertexAttribArray(LC_ATTRIB_TEXCOORD);
-
-		if (mColorEnabled)
-			glDisableVertexAttribArray(LC_ATTRIB_COLOR);
-
-		glVertexAttribPointer(LC_ATTRIB_POSITION, 3, GL_FLOAT, false, 0, nullptr);
-		glVertexAttribPointer(LC_ATTRIB_NORMAL, 4, GL_FLOAT, false, 0, nullptr);
-		glVertexAttribPointer(LC_ATTRIB_TEXCOORD, 2, GL_FLOAT, false, 0, nullptr);
-		glVertexAttribPointer(LC_ATTRIB_COLOR, 4, GL_FLOAT, false, 0, nullptr);
+		DisableVertexAttrib(lcProgramAttrib::Normal);
+		SetVertexAttribPointer(lcProgramAttrib::Normal, 4, GL_BYTE, true, 0, nullptr);
+		DisableVertexAttrib(lcProgramAttrib::TexCoord);
+		SetVertexAttribPointer(lcProgramAttrib::TexCoord, 2, GL_FLOAT, false, 0, nullptr);
+		DisableVertexAttrib(lcProgramAttrib::Color);
+		SetVertexAttribPointer(lcProgramAttrib::Color, 4, GL_FLOAT, false, 0, nullptr);
 	}
 	else
 	{
@@ -867,7 +843,7 @@ void lcContext::SetVertexBuffer(lcVertexBuffer VertexBuffer)
 {
 	if (gSupportsVertexBufferObject)
 	{
-		GLuint VertexBufferObject = VertexBuffer.Object;
+		const GLuint VertexBufferObject = VertexBuffer.Object;
 		mVertexBufferPointer = nullptr;
 
 		if (VertexBufferObject != mVertexBufferObject)
@@ -896,36 +872,59 @@ void lcContext::SetVertexBufferPointer(const void* VertexBuffer)
 	mVertexBufferOffset = (char*)~0;
 }
 
+void lcContext::SetVertexAttribPointer(lcProgramAttrib Attrib, GLint Size, GLenum Type, GLboolean Normalized, GLsizei Stride, const void* Pointer)
+{
+	const int Index = static_cast<int>(Attrib);
+	lcVertexAttribState& State = mVertexAttribState[Index];
+
+	if (State.Size != Size || State.Type != Type || State.Normalized != Normalized || State.Stride != Stride || State.Pointer != Pointer || State.VertexBufferObject != mVertexBufferObject)
+	{
+		glVertexAttribPointer(Index, Size, Type, Normalized, Stride, Pointer);
+
+		State.Size = Size;
+		State.Type = Type;
+		State.Normalized = Normalized;
+		State.Stride = Stride;
+		State.Pointer = Pointer;
+		State.VertexBufferObject = mVertexBufferObject;
+	}
+}
+
+void lcContext::EnableVertexAttrib(lcProgramAttrib Attrib)
+{
+	const int Index = static_cast<int>(Attrib);
+	lcVertexAttribState& State = mVertexAttribState[Index];
+
+	if (!State.Enabled)
+	{
+		glEnableVertexAttribArray(Index);
+		State.Enabled = true;
+	}
+}
+
+void lcContext::DisableVertexAttrib(lcProgramAttrib Attrib)
+{
+	const int Index = static_cast<int>(Attrib);
+	lcVertexAttribState& State = mVertexAttribState[Index];
+
+	if (State.Enabled)
+	{
+		glDisableVertexAttribArray(Index);
+		State.Enabled = false;
+	}
+}
+
 void lcContext::SetVertexFormatPosition(int PositionSize)
 {
-	int VertexSize = PositionSize * sizeof(float);
-	char* VertexBufferPointer = mVertexBufferPointer;
+	const int VertexSize = PositionSize * sizeof(float);
+	const char* VertexBufferPointer = mVertexBufferPointer;
 
 	if (gSupportsShaderObjects)
 	{
-		if (mVertexBufferOffset != mVertexBufferPointer)
-		{
-			glVertexAttribPointer(LC_ATTRIB_POSITION, PositionSize, GL_FLOAT, false, VertexSize, VertexBufferPointer);
-			mVertexBufferOffset = VertexBufferPointer;
-		}
-
-		if (mNormalEnabled)
-		{
-			glDisableVertexAttribArray(LC_ATTRIB_NORMAL);
-			mNormalEnabled = false;
-		}
-
-		if (mTexCoordEnabled)
-		{
-			glDisableVertexAttribArray(LC_ATTRIB_TEXCOORD);
-			mTexCoordEnabled = false;
-		}
-
-		if (mColorEnabled)
-		{
-			glDisableVertexAttribArray(LC_ATTRIB_COLOR);
-			mColorEnabled = false;
-		}
+		SetVertexAttribPointer(lcProgramAttrib::Position, PositionSize, GL_FLOAT, false, VertexSize, VertexBufferPointer);
+		DisableVertexAttrib(lcProgramAttrib::Normal);
+		DisableVertexAttrib(lcProgramAttrib::TexCoord);
+		DisableVertexAttrib(lcProgramAttrib::Color);
 	}
 	else
 	{
@@ -955,72 +954,65 @@ void lcContext::SetVertexFormatPosition(int PositionSize)
 	}
 }
 
-void lcContext::SetVertexFormat(int BufferOffset, int PositionSize, int NormalSize, int TexCoordSize, int ColorSize, bool EnableNormals)
+void lcContext::SetVertexFormatConditional(int BufferOffset)
 {
-	int VertexSize = (PositionSize + TexCoordSize + ColorSize) * sizeof(float) + NormalSize * sizeof(quint32);
-	char* VertexBufferPointer = mVertexBufferPointer + BufferOffset;
+	const int VertexSize = 12 * sizeof(float);
+	const char* VertexBufferPointer = mVertexBufferPointer + BufferOffset;
 
 	if (gSupportsShaderObjects)
 	{
-		if (mVertexBufferOffset != VertexBufferPointer)
-		{
-			glVertexAttribPointer(LC_ATTRIB_POSITION, PositionSize, GL_FLOAT, false, VertexSize, VertexBufferPointer);
-			mVertexBufferOffset = VertexBufferPointer;
-		}
+		SetVertexAttribPointer(lcProgramAttrib::ControlPoint1, 3, GL_FLOAT, false, VertexSize, VertexBufferPointer);
+		EnableVertexAttrib(lcProgramAttrib::ControlPoint1);
+		SetVertexAttribPointer(lcProgramAttrib::ControlPoint2, 3, GL_FLOAT, false, VertexSize, VertexBufferPointer + 3 * sizeof(float));
+		EnableVertexAttrib(lcProgramAttrib::ControlPoint2);
+		SetVertexAttribPointer(lcProgramAttrib::ControlPoint3, 3, GL_FLOAT, false, VertexSize, VertexBufferPointer + 6 * sizeof(float));
+		EnableVertexAttrib(lcProgramAttrib::ControlPoint3);
+		SetVertexAttribPointer(lcProgramAttrib::ControlPoint4, 3, GL_FLOAT, false, VertexSize, VertexBufferPointer + 9 * sizeof(float));
+		EnableVertexAttrib(lcProgramAttrib::ControlPoint4);
+	}
+}
 
-		int Offset = PositionSize * sizeof(float);
+void lcContext::SetVertexFormat(int BufferOffset, int PositionSize, int NormalSize, int TexCoordSize, int ColorSize, bool EnableNormals)
+{
+	const int VertexSize = (PositionSize + TexCoordSize) * sizeof(float) + NormalSize * sizeof(quint32) + ColorSize;
+	const char* VertexBufferPointer = mVertexBufferPointer + BufferOffset;
+
+	if (gSupportsShaderObjects)
+	{
+		int Offset = 0;
+
+		SetVertexAttribPointer(lcProgramAttrib::Position, PositionSize, GL_FLOAT, false, VertexSize, VertexBufferPointer);
+		EnableVertexAttrib(lcProgramAttrib::Position);
+
+		Offset += PositionSize * sizeof(float);
 
 		if (NormalSize && EnableNormals)
 		{
-			glVertexAttribPointer(LC_ATTRIB_NORMAL, 4, GL_BYTE, true, VertexSize, VertexBufferPointer + Offset);
-
-			if (!mNormalEnabled)
-			{
-				glEnableVertexAttribArray(LC_ATTRIB_NORMAL);
-				mNormalEnabled = true;
-			}
-		}
-		else if (mNormalEnabled)
-		{
-			glDisableClientState(GL_NORMAL_ARRAY);
-			mNormalEnabled = false;
+			SetVertexAttribPointer(lcProgramAttrib::Normal, 4, GL_BYTE, true, VertexSize, VertexBufferPointer + Offset);
+			EnableVertexAttrib(lcProgramAttrib::Normal);
 		}
+		else
+			DisableVertexAttrib(lcProgramAttrib::Normal);
 
 		Offset += NormalSize * sizeof(quint32);
 
 		if (TexCoordSize)
 		{
-			glVertexAttribPointer(LC_ATTRIB_TEXCOORD, TexCoordSize, GL_FLOAT, false, VertexSize, VertexBufferPointer + Offset);
-
-			if (!mTexCoordEnabled)
-			{
-				glEnableVertexAttribArray(LC_ATTRIB_TEXCOORD);
-				mTexCoordEnabled = true;
-			}
-
-			Offset += 2 * sizeof(float);
-		}
-		else if (mTexCoordEnabled)
-		{
-			glDisableVertexAttribArray(LC_ATTRIB_TEXCOORD);
-			mTexCoordEnabled = false;
+			SetVertexAttribPointer(lcProgramAttrib::TexCoord, TexCoordSize, GL_FLOAT, false, VertexSize, VertexBufferPointer + Offset);
+			EnableVertexAttrib(lcProgramAttrib::TexCoord);
 		}
+		else
+			DisableVertexAttrib(lcProgramAttrib::TexCoord);
+
+		Offset += TexCoordSize * sizeof(float);
 
 		if (ColorSize)
 		{
-			glVertexAttribPointer(LC_ATTRIB_COLOR, ColorSize, GL_FLOAT, false, VertexSize, VertexBufferPointer + Offset);
-
-			if (!mColorEnabled)
-			{
-				glEnableVertexAttribArray(LC_ATTRIB_COLOR);
-				mColorEnabled = true;
-			}
-		}
-		else if (mColorEnabled)
-		{
-			glDisableVertexAttribArray(LC_ATTRIB_COLOR);
-			mColorEnabled = false;
+			SetVertexAttribPointer(lcProgramAttrib::Color, ColorSize, GL_UNSIGNED_BYTE, true, VertexSize, VertexBufferPointer + Offset);
+			EnableVertexAttrib(lcProgramAttrib::Color);
 		}
+		else
+			DisableVertexAttrib(lcProgramAttrib::Color);
 	}
 	else
 	{
@@ -1101,7 +1093,7 @@ void lcContext::SetIndexBuffer(lcIndexBuffer IndexBuffer)
 {
 	if (gSupportsVertexBufferObject)
 	{
-		GLuint IndexBufferObject = IndexBuffer.Object;
+		const GLuint IndexBufferObject = IndexBuffer.Object;
 		mIndexBufferPointer = nullptr;
 
 		if (IndexBufferObject != mIndexBufferObject)
@@ -1129,12 +1121,12 @@ void lcContext::SetIndexBufferPointer(const void* IndexBuffer)
 
 void lcContext::BindMesh(const lcMesh* Mesh)
 {
-	lcPiecesLibrary* Library = lcGetPiecesLibrary();
+	const lcPiecesLibrary* const Library = lcGetPiecesLibrary();
 
 	if (Mesh->mVertexCacheOffset != -1)
 	{
-		GLuint VertexBufferObject = Library->mVertexBuffer.Object;
-		GLuint IndexBufferObject = Library->mIndexBuffer.Object;
+		const GLuint VertexBufferObject = Library->mVertexBuffer.Object;
+		const GLuint IndexBufferObject = Library->mIndexBuffer.Object;
 
 		if (VertexBufferObject != mVertexBufferObject)
 		{
@@ -1175,7 +1167,7 @@ void lcContext::FlushState()
 {
 	if (gSupportsShaderObjects)
 	{
-		const lcProgram& Program = mPrograms[mMaterialType];
+		const lcProgram& Program = mPrograms[static_cast<int>(mMaterialType)];
 
 		if (mWorldMatrixDirty || mViewMatrixDirty || mProjectionMatrixDirty)
 		{
@@ -1193,7 +1185,7 @@ void lcContext::FlushState()
 
 			if (mViewMatrixDirty)
 			{
-				lcMatrix44 InverseViewMatrix = lcMatrix44AffineInverse(mViewMatrix);
+				const lcMatrix44 InverseViewMatrix = lcMatrix44AffineInverse(mViewMatrix);
 				lcVector3 ViewPosition = lcMul30(-mViewMatrix.GetTranslation(), InverseViewMatrix);
 
 				if (Program.LightPositionLocation != -1)
@@ -1220,10 +1212,7 @@ void lcContext::FlushState()
 
 		if (mHighlightParamsDirty && Program.HighlightParamsLocation != -1)
 		{
-			lcMatrix44 InverseViewMatrix = lcMatrix44AffineInverse(mViewMatrix);
-			mHighlightParams[4] = InverseViewMatrix[2];
-
-			glUniform4fv(Program.HighlightParamsLocation, 5, mHighlightParams[0]);
+			glUniform4fv(Program.HighlightParamsLocation, 4, mHighlightParams[0]);
 			mHighlightParamsDirty = false;
 		}
 	}
