@@ -8,7 +8,7 @@
 class lcVertexBuffer
 {
 public:
-	lcVertexBuffer()
+	constexpr lcVertexBuffer()
 		: Pointer(nullptr)
 	{
 	}
@@ -28,7 +28,7 @@ public:
 class lcIndexBuffer
 {
 public:
-	lcIndexBuffer()
+	constexpr lcIndexBuffer()
 		: Pointer(nullptr)
 	{
 	}
@@ -45,24 +45,30 @@ public:
 	};
 };
 
-enum lcMaterialType
+enum class lcMaterialType
 {
-	LC_MATERIAL_UNLIT_COLOR,
-	LC_MATERIAL_UNLIT_TEXTURE_MODULATE,
-	LC_MATERIAL_UNLIT_TEXTURE_DECAL,
-	LC_MATERIAL_UNLIT_VERTEX_COLOR,
-	LC_MATERIAL_UNLIT_VIEW_SPHERE,
-	LC_MATERIAL_FAKELIT_COLOR,
-	LC_MATERIAL_FAKELIT_TEXTURE_DECAL,
-	LC_NUM_MATERIALS
+	UnlitColor,
+	UnlitColorConditional,
+	UnlitTextureModulate,
+	UnlitTextureDecal,
+	UnlitVertexColor,
+	UnlitViewSphere,
+	FakeLitColor,
+	FakeLitTextureDecal,
+	Count
 };
 
-enum lcProgramAttrib
+enum class lcProgramAttrib
 {
-	LC_ATTRIB_POSITION,
-	LC_ATTRIB_NORMAL,
-	LC_ATTRIB_TEXCOORD,
-	LC_ATTRIB_COLOR
+	Position,
+	Normal,
+	TexCoord,
+	Color,
+	ControlPoint1 = 0,
+	ControlPoint2,
+	ControlPoint3,
+	ControlPoint4,
+	Count
 };
 
 struct lcProgram
@@ -76,49 +82,57 @@ struct lcProgram
 	GLint HighlightParamsLocation;
 };
 
-class lcFramebuffer
+enum class lcPolygonOffset
 {
-public:
-	lcFramebuffer()
-	{
-	}
-
-	lcFramebuffer(int Width, int Height)
-		: mWidth(Width), mHeight(Height)
-	{
-	}
-
-	bool IsValid() const
-	{
-		return mObject != 0;
-	}
-
-	GLuint mObject = 0;
-	GLuint mColorTexture = 0;
-	GLuint mDepthRenderbuffer = 0;
-	int mWidth = 0;
-	int mHeight = 0;
+	None,
+	Opaque,
+	Translucent
 };
 
-enum lcPolygonOffset
+enum class lcDepthFunction
 {
-	LC_POLYGON_OFFSET_NONE,
-	LC_POLYGON_OFFSET_OPAQUE,
-	LC_POLYGON_OFFSET_TRANSLUCENT
+	LessEqual,
+	Always
 };
 
-class lcContext
+struct lcVertexAttribState
+{
+	GLint Size = 0;
+	GLenum Type = 0;
+	GLboolean Normalized = 0;
+	bool Enabled = 0;
+	GLsizei Stride = 0;
+	const void* Pointer = nullptr;
+	GLuint VertexBufferObject = 0;
+};
+
+class lcContext : protected QOpenGLFunctions
 {
 public:
 	lcContext();
 	~lcContext();
 
-	static void CreateResources();
-	static void DestroyResources();
+	lcContext(const lcContext&) = delete;
+	lcContext& operator=(const lcContext&) = delete;
+
+	static bool InitializeRenderer();
+	static void ShutdownRenderer();
+	static lcContext* GetGlobalOffscreenContext();
+
+	void CreateResources();
+	void DestroyResources();
 
 	void SetDefaultState();
 	void ClearResources();
 
+	void MakeCurrent();
+
+	void SetGLContext(QOpenGLContext* GLContext, QOpenGLWidget* Widget);
+	void SetOffscreenContext();
+
+	void ClearColorAndDepth(const lcVector4& ClearColor);
+	void ClearDepth();
+
 	void SetWorldMatrix(const lcMatrix44& WorldMatrix)
 	{
 		mWorldMatrix = WorldMatrix;
@@ -147,12 +161,12 @@ public:
 	void SetMaterial(lcMaterialType MaterialType);
 	void SetViewport(int x, int y, int Width, int Height);
 	void SetPolygonOffset(lcPolygonOffset PolygonOffset);
+	void SetDepthWrite(bool Enable);
+	void SetDepthFunction(lcDepthFunction DepthFunction);
+	void EnableCullFace(bool Enable);
 	void SetLineWidth(float LineWidth);
 	void SetSmoothShading(bool Smooth);
-	void BeginTranslucent();
-	void EndTranslucent();
 	void BindTexture2D(GLuint Texture);
-	void BindTexture2DMS(GLuint Texture);
 	void BindTextureCubeMap(GLuint Texture);
 	void UnbindTexture2D(GLuint Texture);
 	void UnbindTextureCubeMap(GLuint Texture);
@@ -175,23 +189,11 @@ public:
 	void SetColor(float Red, float Green, float Blue, float Alpha);
 	void SetColorIndex(int ColorIndex);
 	void SetColorIndexTinted(int ColorIndex, lcInterfaceColor InterfaceColor, float Weight);
+	void SetColorIndexTinted(int ColorIndex, const lcVector4& Tint);
 	void SetEdgeColorIndex(int ColorIndex);
+	void SetEdgeColorIndexTinted(int ColorIndex, const lcVector4& Tint);
 	void SetInterfaceColor(lcInterfaceColor InterfaceColor);
 
-	void ClearFramebuffer();
-	lcFramebuffer CreateFramebuffer(int Width, int Height, bool Depth, bool Multisample);
-	void DestroyFramebuffer(lcFramebuffer& Framebuffer);
-	void BindFramebuffer(GLuint FramebufferObject);
-	void BindFramebuffer(const lcFramebuffer& Framebuffer)
-	{
-		BindFramebuffer(Framebuffer.mObject);
-	}
-
-	std::pair<lcFramebuffer, lcFramebuffer> CreateRenderFramebuffer(int Width, int Height);
-	void DestroyRenderFramebuffer(std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer);
-	QImage GetRenderFramebufferImage(const std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer);
-	void GetRenderFramebufferImage(const std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer, quint8* Buffer);
-
 	lcVertexBuffer CreateVertexBuffer(int Size, const void* Data);
 	void DestroyVertexBuffer(lcVertexBuffer& VertexBuffer);
 	lcIndexBuffer CreateIndexBuffer(int Size, const void* Data);
@@ -207,30 +209,45 @@ public:
 
 	void SetVertexFormat(int BufferOffset, int PositionSize, int NormalSize, int TexCoordSize, int ColorSize, bool EnableNormals);
 	void SetVertexFormatPosition(int PositionSize);
+	void SetVertexFormatConditional(int BufferOffset);
 	void DrawPrimitives(GLenum Mode, GLint First, GLsizei Count);
 	void DrawIndexedPrimitives(GLenum Mode, GLsizei Count, GLenum Type, int Offset);
 
 	void BindMesh(const lcMesh* Mesh);
 
 protected:
-	static void CreateShaderPrograms();
+	static bool CreateOffscreenContext();
+	static void DestroyOffscreenContext();
+
+	void CreateShaderPrograms();
 	void FlushState();
 
+	void SetVertexAttribPointer(lcProgramAttrib Attrib, GLint Size, GLenum Type, GLboolean Normalized, GLsizei Stride, const void* Pointer);
+	void EnableVertexAttrib(lcProgramAttrib Attrib);
+	void DisableVertexAttrib(lcProgramAttrib Attrib);
+
+	QOpenGLWidget* mWidget = nullptr;
+	QOpenGLContext* mContext = nullptr;
+
 	GLuint mVertexBufferObject;
 	GLuint mIndexBufferObject;
-	char* mVertexBufferPointer;
-	char* mIndexBufferPointer;
-	char* mVertexBufferOffset;
+	const char* mVertexBufferPointer;
+	const char* mIndexBufferPointer;
+	const char* mVertexBufferOffset;
 
 	lcMaterialType mMaterialType;
 	bool mNormalEnabled;
 	bool mTexCoordEnabled;
 	bool mColorEnabled;
 
+	lcVertexAttribState mVertexAttribState[static_cast<int>(lcProgramAttrib::Count)];
+
 	GLuint mTexture2D;
-	GLuint mTexture2DMS;
 	GLuint mTextureCubeMap;
 	lcPolygonOffset mPolygonOffset;
+	bool mDepthWrite;
+	lcDepthFunction mDepthFunction;
+	bool mCullFace;
 	float mLineWidth;
 	int mMatrixMode;
 	bool mTextureEnabled;
@@ -240,7 +257,7 @@ protected:
 	lcMatrix44 mViewMatrix;
 	lcMatrix44 mProjectionMatrix;
 	lcMatrix44 mViewProjectionMatrix;
-	lcVector4 mHighlightParams[5];
+	lcVector4 mHighlightParams[4];
 	bool mColorDirty;
 	bool mWorldMatrixDirty;
 	bool mViewMatrixDirty;
@@ -248,9 +265,11 @@ protected:
 	bool mViewProjectionMatrixDirty;
 	bool mHighlightParamsDirty;
 
-	GLuint mFramebufferObject;
+	static std::unique_ptr<QOpenGLContext> mOffscreenContext;
+	static std::unique_ptr<QOffscreenSurface> mOffscreenSurface;
+	static std::unique_ptr<lcContext> mGlobalOffscreenContext;
 
-	static lcProgram mPrograms[LC_NUM_MATERIALS];
+	static lcProgram mPrograms[static_cast<int>(lcMaterialType::Count)];
 
 	Q_DECLARE_TR_FUNCTIONS(lcContext);
 };
