@@ -1,35 +1,25 @@
 #pragma once
 
-#include "lc_basewindow.h"
+#include "lc_application.h"
+#include "lc_shortcuts.h"
 #include "lc_array.h"
 #include "lc_commands.h"
 #include "lc_model.h"
 
-class View;
 class lcPartSelectionWidget;
+class lcPreviewDockWidget;
 class PiecePreview;
-class lcQGLWidget;
 class lcQPartsTree;
 class lcQColorList;
 class lcQPropertiesTree;
 class lcTimelineWidget;
+class lcElidedLabel;
 #ifdef QT_NO_PRINTER
 class QPrinter;
 #endif
 
 #define LC_MAX_RECENT_FILES 4
 
-struct lcSearchOptions
-{
-	bool SearchValid;
-	bool MatchInfo;
-	bool MatchColor;
-	bool MatchName;
-	PieceInfo* Info;
-	int ColorIndex;
-	char Name[256];
-};
-
 class lcTabBar : public QTabBar
 {
 public:
@@ -39,25 +29,12 @@ public:
 	}
 
 protected:
-	virtual void mousePressEvent(QMouseEvent* Event) override;
-	virtual void mouseReleaseEvent(QMouseEvent* Event) override;
+	void mousePressEvent(QMouseEvent* Event) override;
+	void mouseReleaseEvent(QMouseEvent* Event) override;
 
 	int mMousePressTab;
 };
 
-class lcTabWidget : public QTabWidget
-{
-public:
-	lcTabWidget();
-
-#if (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
-	QTabBar* tabBar()
-	{
-		return QTabWidget::tabBar();
-	}
-#endif
-};
-
 class lcModelTabWidget : public QWidget
 {
 	Q_OBJECT
@@ -69,9 +46,6 @@ public:
 		mActiveView = nullptr;
 	}
 
-	void ResetLayout();
-	void Clear();
-
 	QWidget* GetAnyViewWidget()
 	{
 		QWidget* Widget = layout()->itemAt(0)->widget();
@@ -82,27 +56,20 @@ public:
 		return Widget;
 	}
 
-	View* GetActiveView() const
+	lcView* GetActiveView() const
 	{
 		return mActiveView;
 	}
 
-	void SetActiveView(View* ActiveView)
+	void SetActiveView(lcView* ActiveView)
 	{
 		mActiveView = ActiveView;
 	}
 
-	void AddView(View* View)
-	{
-		mViews.Add(View);
-	}
-
-	void RemoveView(View* View)
+	void RemoveView(lcView* View)
 	{
 		if (View == mActiveView)
 			mActiveView = nullptr;
-
-		mViews.Remove(View);
 	}
 
 	lcModel* GetModel() const
@@ -110,20 +77,9 @@ public:
 		return mModel;
 	}
 
-	void SetModel(lcModel* Model)
-	{
-		mModel = Model;
-	}
-
-	const lcArray<View*>* GetViews() const
-	{
-		return &mViews;
-	}
-
 protected:
 	lcModel* mModel;
-	View* mActiveView;
-	lcArray<View*> mViews;
+	lcView* mActiveView;
 };
 
 class lcMainWindow : public QMainWindow
@@ -191,6 +147,11 @@ public:
 		return mRelativeTransform;
 	}
 
+	bool GetSeparateTransform() const
+	{
+		return mLocalTransform;
+	}
+
 	lcSelectionMode GetSelectionMode() const
 	{
 		return mSelectionMode;
@@ -201,45 +162,31 @@ public:
 		return mCurrentPieceInfo;
 	}
 
-	View* GetActiveView() const
+	lcView* GetActiveView() const
 	{
-		lcModelTabWidget* CurrentTab = mModelTabWidget ? (lcModelTabWidget*)mModelTabWidget->currentWidget() : nullptr;
+		const lcModelTabWidget* const CurrentTab = mModelTabWidget ? (lcModelTabWidget*)mModelTabWidget->currentWidget() : nullptr;
 		return CurrentTab ? CurrentTab->GetActiveView() : nullptr;
 	}
 
 	lcModel* GetActiveModel() const;
+	lcModelTabWidget* GetTabForView(lcView* View) const;
 
 	lcModel* GetCurrentTabModel() const
 	{
-		lcModelTabWidget* CurrentTab = (lcModelTabWidget*)mModelTabWidget->currentWidget();
+		const lcModelTabWidget* const CurrentTab = (lcModelTabWidget*)mModelTabWidget->currentWidget();
 		return CurrentTab ? CurrentTab->GetModel() : nullptr;
 	}
 
-	const lcArray<View*>* GetViewsForModel(lcModel* Model) const
-	{
-		lcModelTabWidget* TabWidget = GetTabWidgetForModel(Model);
-		return TabWidget ? TabWidget->GetViews() : nullptr;
-	}
-
-	lcModelTabWidget* GetTabForView(View* View) const
-	{
-		for (int TabIdx = 0; TabIdx < mModelTabWidget->count(); TabIdx++)
-		{
-			lcModelTabWidget* TabWidget = (lcModelTabWidget*)mModelTabWidget->widget(TabIdx);
-
-			int ViewIndex = TabWidget->GetViews()->FindIndex(View);
-			if (ViewIndex != -1)
-				return TabWidget;
-		}
-
-		return nullptr;
-	}
-
 	lcPartSelectionWidget* GetPartSelectionWidget() const
 	{
 		return mPartSelectionWidget;
 	}
 
+	lcPreviewDockWidget* GetPreviewWidget() const
+	{
+		return mPreviewWidget;
+	}
+
 	QMenu* GetToolsMenu() const
 	{
 		return mToolsMenu;
@@ -271,10 +218,8 @@ public:
 	void CloseCurrentModelTab();
 	void SetCurrentModelTab(lcModel* Model);
 	void ResetCameras();
-	void AddView(View* View);
-	void RemoveView(View* View);
-	void SetActiveView(View* ActiveView);
-	void UpdateAllViews();
+	void AddView(lcView* View);
+	void RemoveView(lcView* View);
 
 	void SetTool(lcTool Tool);
 	void SetTransformType(lcTransformType TransformType);
@@ -285,12 +230,18 @@ public:
 	void SetMoveZSnapIndex(int Index);
 	void SetAngleSnapIndex(int Index);
 	void SetRelativeTransform(bool RelativeTransform);
+	void SetSeparateTransform(bool SelectionTransform);
 	void SetCurrentPieceInfo(PieceInfo* Info);
 	void SetShadingMode(lcShadingMode ShadingMode);
 	void SetSelectionMode(lcSelectionMode SelectionMode);
+	void ToggleViewSphere();
+	void ToggleAxisIcon();
+	void ToggleGrid();
+	void ToggleFadePreviousSteps();
 
 	void NewProject();
 	bool OpenProject(const QString& FileName);
+	void OpenRecentProject(int RecentFileIndex);
 	void MergeProject();
 	void ImportLDD();
 	void ImportInventory();
@@ -320,7 +271,6 @@ public:
 	void UpdateSnap();
 	void UpdateColor();
 	void UpdateUndoRedo(const QString& UndoText, const QString& RedoText);
-	void UpdateCurrentCamera(int CameraIndex);
 	void UpdatePerspective();
 	void UpdateCameraMenu();
 	void UpdateShadingMode();
@@ -336,43 +286,54 @@ public:
 
 	QString mRecentFiles[LC_MAX_RECENT_FILES];
 	int mColorIndex;
-	lcSearchOptions mSearchOptions;
 	QAction* mActions[LC_NUM_COMMANDS];
 
 public slots:
 	void ProjectFileChanged(const QString& Path);
+	void PreviewPiece(const QString& PartId, int ColorCode, bool ShowPreview);
+	void TogglePreviewWidget(bool Visible);
 
 protected slots:
+	void ViewFocusReceived();
+	void ViewCameraChanged();
+	void UpdateDockWidgetActions();
 	void UpdateGamepads();
 	void ModelTabContextMenuRequested(const QPoint& Point);
 	void ModelTabCloseOtherTabs();
-	void ModelTabResetViews();
 	void ModelTabClosed(int Index);
 	void ModelTabChanged(int Index);
 	void ClipboardChanged();
 	void ActionTriggered();
 	void ColorChanged(int ColorIndex);
+	void ColorButtonClicked();
 	void Print(QPrinter* Printer);
+	void EnableWindowFlags(bool);
 
 protected:
-	void closeEvent(QCloseEvent *event);
-	void dragEnterEvent(QDragEnterEvent* Event);
-	void dropEvent(QDropEvent* Event);
-	QMenu* createPopupMenu();
+	void closeEvent(QCloseEvent *event) override;
+	void dragEnterEvent(QDragEnterEvent* Event) override;
+	void dropEvent(QDropEvent* Event) override;
+	QMenu* createPopupMenu() override;
 
 	void CreateActions();
 	void CreateMenus();
 	void CreateToolBars();
 	void CreateStatusBar();
+	lcView* CreateView(lcModel* Model);
+	void SetActiveView(lcView* ActiveView);
+	void ToggleDockWidget(QWidget* DockWidget);
 	void SplitView(Qt::Orientation Orientation);
-	void ShowSearchDialog();
 	void ShowUpdatesDialog();
 	void ShowAboutDialog();
 	void ShowHTMLDialog();
 	void ShowRenderDialog();
+	void ShowInstructionsDialog();
 	void ShowPrintDialog();
+	void CreatePreviewWidget();
 
-	lcModelTabWidget* GetTabWidgetForModel(lcModel* Model) const
+	bool OpenProjectFile(const QString& FileName);
+
+	lcModelTabWidget* GetTabWidgetForModel(const lcModel* Model) const
 	{
 		for (int TabIdx = 0; TabIdx < mModelTabWidget->count(); TabIdx++)
 		{
@@ -397,16 +358,18 @@ protected:
 	int mMoveZSnapIndex;
 	int mAngleSnapIndex;
 	bool mRelativeTransform;
+	bool mLocalTransform;
 	PieceInfo* mCurrentPieceInfo;
 	lcSelectionMode mSelectionMode;
 	int mModelTabWidgetContextMenuIndex;
 
 	QAction* mActionFileRecentSeparator;
 
-	lcTabWidget* mModelTabWidget;
+	QTabWidget* mModelTabWidget;
 	QToolBar* mStandardToolBar;
 	QToolBar* mToolsToolBar;
 	QToolBar* mTimeToolBar;
+	QDockWidget* mPreviewToolBar;
 	QDockWidget* mPartsToolBar;
 	QDockWidget* mColorsToolBar;
 	QDockWidget* mPropertiesToolBar;
@@ -414,17 +377,20 @@ protected:
 
 	lcPartSelectionWidget* mPartSelectionWidget;
 	lcQColorList* mColorList;
+	QToolButton* mColorButton;
 	lcQPropertiesTree* mPropertiesWidget;
 	lcTimelineWidget* mTimelineWidget;
 	QLineEdit* mTransformXEdit;
 	QLineEdit* mTransformYEdit;
 	QLineEdit* mTransformZEdit;
+	lcPreviewDockWidget* mPreviewWidget;
 
-	QLabel* mStatusBarLabel;
+	lcElidedLabel* mStatusBarLabel;
 	QLabel* mStatusPositionLabel;
 	QLabel* mStatusSnapLabel;
 	QLabel* mStatusTimeLabel;
 
+	QMenu* mTransformMenu;
 	QMenu* mToolsMenu;
 	QMenu* mViewpointMenu;
 	QMenu* mCameraMenu;
