@@ -2,14 +2,10 @@
 
 #include "lc_math.h"
 
-#define LC_MESH_FILE_ID      LC_FOURCC('M', 'E', 'S', 'H')
-#define LC_MESH_FILE_VERSION 0x0114
-
 enum lcMeshPrimitiveType
 {
 	LC_MESH_LINES = 0x01,
 	LC_MESH_TRIANGLES = 0x02,
-	LC_MESH_TEXTURED_LINES = 0x04,
 	LC_MESH_TEXTURED_TRIANGLES = 0x08,
 	LC_MESH_CONDITIONAL_LINES = 0x10,
 	LC_MESH_NUM_PRIMITIVE_TYPES
@@ -28,6 +24,14 @@ struct lcVertexTextured
 	lcVector2 TexCoord;
 };
 
+struct lcVertexConditional
+{
+	lcVector3 Position1;
+	lcVector3 Position2;
+	lcVector3 Position3;
+	lcVector3 Position4;
+};
+
 struct lcMeshSection
 {
 	int ColorIndex;
@@ -35,6 +39,8 @@ struct lcMeshSection
 	int NumIndices;
 	lcMeshPrimitiveType PrimitiveType;
 	lcTexture* Texture;
+	lcBoundingBox BoundingBox;
+	float Radius;
 };
 
 struct lcMeshLod
@@ -50,13 +56,29 @@ enum
 	LC_NUM_MESH_LODS
 };
 
+enum class lcMeshFlag
+{
+	HasDefault     = 0x01, // Mesh has triangles using the default color
+	HasSolid       = 0x02, // Mesh has triangles using a solid color
+	HasTranslucent = 0x04, // Mesh has triangles using a translucent color
+	HasLines       = 0x08, // Mesh has lines
+	HasTexture     = 0x10, // Mesh has sections using textures
+	HasStyleStud   = 0x20  // Mesh has a stud that can have a logo applied
+};
+
+Q_DECLARE_FLAGS(lcMeshFlags, lcMeshFlag)
+Q_DECLARE_OPERATORS_FOR_FLAGS(lcMeshFlags)
+
 class lcMesh
 {
 public:
 	lcMesh();
 	~lcMesh();
 
-	void Create(quint16 NumSections[LC_NUM_MESH_LODS], int NumVertices, int NumTexturedVertices, int NumIndices);
+	lcMesh(const lcMesh&) = delete;
+	lcMesh& operator=(const lcMesh&) = delete;
+
+	void Create(quint16 (&NumSections)[LC_NUM_MESH_LODS], int VertexCount, int TexturedVertexCount, int ConditionalVertexCount, int IndexCount);
 	void CreateBox();
 
 	bool FileLoad(lcMemFile& File);
@@ -75,14 +97,30 @@ public:
 	bool MinIntersectDist(const lcVector3& Start, const lcVector3& End, float& MinDist);
 
 	template<typename IndexType>
-	bool IntersectsPlanes(const lcVector4 Planes[6]);
-	bool IntersectsPlanes(const lcVector4 Planes[6]);
+	bool IntersectsPlanes(const lcVector4 (&Planes)[6]);
+	bool IntersectsPlanes(const lcVector4 (&Planes)[6]);
 
 	int GetLodIndex(float Distance) const;
 
+	const lcVertex* GetVertexData() const
+	{
+		return static_cast<lcVertex*>(mVertexData);
+	}
+
+	const lcVertexTextured* GetTexturedVertexData() const
+	{
+		return reinterpret_cast<lcVertexTextured*>(static_cast<char*>(mVertexData) + mNumVertices * sizeof(lcVertex));
+	}
+
+	const lcVertexConditional* GetConditionalVertexData() const
+	{
+		return reinterpret_cast<lcVertexConditional*>(static_cast<char*>(mVertexData) + mNumVertices * sizeof(lcVertex) + mNumTexturedVertices * sizeof(lcVertexTextured));
+	}
+
 	lcMeshLod mLods[LC_NUM_MESH_LODS];
 	lcBoundingBox mBoundingBox;
 	float mRadius;
+	lcMeshFlags mFlags;
 
 	void* mVertexData;
 	int mVertexDataSize;
@@ -93,27 +131,8 @@ public:
 
 	int mNumVertices;
 	int mNumTexturedVertices;
+	int mConditionalVertexCount;
 	int mIndexType;
 };
 
-enum class lcRenderMeshState : int
-{
-	NORMAL,
-	SELECTED,
-	FOCUSED,
-	DISABLED,
-	HIGHLIGHT
-};
-
-struct lcRenderMesh
-{
-	lcMatrix44 WorldMatrix;
-	lcMesh* Mesh;
-	float Distance;
-	int ColorIndex;
-	int LodIndex;
-	lcRenderMeshState State;
-};
-
-extern lcMesh* gPlaceholderMesh;
 
