@@ -5,24 +5,48 @@ enum LC_PROFILE_KEY
 	// Settings.
 	LC_PROFILE_FIXED_AXES,
 	LC_PROFILE_LINE_WIDTH,
+	LC_PROFILE_ALLOW_LOD,
+	LC_PROFILE_LOD_DISTANCE,
+	LC_PROFILE_FADE_STEPS,
+	LC_PROFILE_FADE_STEPS_COLOR,
+	LC_PROFILE_HIGHLIGHT_NEW_PARTS,
+	LC_PROFILE_HIGHLIGHT_NEW_PARTS_COLOR,
 	LC_PROFILE_SHADING_MODE,
+	LC_PROFILE_BACKGROUND_GRADIENT,
+	LC_PROFILE_BACKGROUND_COLOR,
+	LC_PROFILE_GRADIENT_COLOR_TOP,
+	LC_PROFILE_GRADIENT_COLOR_BOTTOM,
 	LC_PROFILE_DRAW_AXES,
+	LC_PROFILE_DRAW_AXES_LOCATION,
+	LC_PROFILE_AXES_COLOR,
+	LC_PROFILE_TEXT_COLOR,
+	LC_PROFILE_MARQUEE_BORDER_COLOR,
+	LC_PROFILE_MARQUEE_FILL_COLOR,
+	LC_PROFILE_OVERLAY_COLOR,
+	LC_PROFILE_ACTIVE_VIEW_COLOR,
+	LC_PROFILE_INACTIVE_VIEW_COLOR,
 	LC_PROFILE_DRAW_EDGE_LINES,
+	LC_PROFILE_DRAW_CONDITIONAL_LINES,
 	LC_PROFILE_GRID_STUDS,
 	LC_PROFILE_GRID_STUD_COLOR,
 	LC_PROFILE_GRID_LINES,
 	LC_PROFILE_GRID_LINE_SPACING,
 	LC_PROFILE_GRID_LINE_COLOR,
+	LC_PROFILE_GRID_ORIGIN,
 	LC_PROFILE_ANTIALIASING_SAMPLES,
+	LC_PROFILE_VIEW_SPHERE_ENABLED,
 	LC_PROFILE_VIEW_SPHERE_LOCATION,
 	LC_PROFILE_VIEW_SPHERE_SIZE,
 	LC_PROFILE_VIEW_SPHERE_COLOR,
 	LC_PROFILE_VIEW_SPHERE_TEXT_COLOR,
 	LC_PROFILE_VIEW_SPHERE_HIGHLIGHT_COLOR,
 
+	LC_PROFILE_LANGUAGE,
+	LC_PROFILE_COLOR_THEME,
 	LC_PROFILE_CHECK_UPDATES,
 	LC_PROFILE_PROJECTS_PATH,
 	LC_PROFILE_PARTS_LIBRARY,
+	LC_PROFILE_PART_PALETTES,
 	LC_PROFILE_MINIFIG_SETTINGS,
 	LC_PROFILE_COLOR_CONFIG,
 	LC_PROFILE_KEYBOARD_SHORTCUTS,
@@ -33,44 +57,47 @@ enum LC_PROFILE_KEY
 	LC_PROFILE_RECENT_FILE3,
 	LC_PROFILE_RECENT_FILE4,
 	LC_PROFILE_AUTOLOAD_MOSTRECENT,
+	LC_PROFILE_RESTORE_TAB_LAYOUT,
 	LC_PROFILE_AUTOSAVE_INTERVAL,
 	LC_PROFILE_MOUSE_SENSITIVITY,
 	LC_PROFILE_IMAGE_WIDTH,
 	LC_PROFILE_IMAGE_HEIGHT,
 	LC_PROFILE_IMAGE_EXTENSION,
-	LC_PROFILE_PRINT_ROWS,
-	LC_PROFILE_PRINT_COLUMNS,
 	LC_PROFILE_PARTS_LIST_ICONS,
 	LC_PROFILE_PARTS_LIST_NAMES,
 	LC_PROFILE_PARTS_LIST_COLOR,
 	LC_PROFILE_PARTS_LIST_DECORATED,
+	LC_PROFILE_PARTS_LIST_ALIASES,
 	LC_PROFILE_PARTS_LIST_LISTMODE,
+	LC_PROFILE_STUD_STYLE,
 
 	// Defaults for new projects.
 	LC_PROFILE_DEFAULT_AUTHOR_NAME,
-	LC_PROFILE_DEFAULT_FLOOR_COLOR,
-	LC_PROFILE_DEFAULT_FLOOR_TEXTURE,
 	LC_PROFILE_DEFAULT_AMBIENT_COLOR,
-	LC_PROFILE_DEFAULT_BACKGROUND_TYPE,
-	LC_PROFILE_DEFAULT_BACKGROUND_COLOR,
-	LC_PROFILE_DEFAULT_GRADIENT_COLOR1,
-	LC_PROFILE_DEFAULT_GRADIENT_COLOR2,
-	LC_PROFILE_DEFAULT_BACKGROUND_TEXTURE,
-	LC_PROFILE_DEFAULT_BACKGROUND_TILE,
 
 	// Exporters.
 	LC_PROFILE_HTML_OPTIONS,
 	LC_PROFILE_HTML_IMAGE_OPTIONS,
 	LC_PROFILE_HTML_IMAGE_WIDTH,
 	LC_PROFILE_HTML_IMAGE_HEIGHT,
-	LC_PROFILE_HTML_PARTS_COLOR,
-	LC_PROFILE_HTML_PARTS_WIDTH,
-	LC_PROFILE_HTML_PARTS_HEIGHT,
 	LC_PROFILE_POVRAY_PATH,
 	LC_PROFILE_POVRAY_LGEO_PATH,
 	LC_PROFILE_POVRAY_WIDTH,
 	LC_PROFILE_POVRAY_HEIGHT,
 
+	LC_PROFILE_PREVIEW_VIEW_SPHERE_ENABLED,
+	LC_PROFILE_PREVIEW_VIEW_SPHERE_SIZE,
+	LC_PROFILE_PREVIEW_VIEW_SPHERE_LOCATION,
+	LC_PROFILE_PREVIEW_DRAW_AXES,
+
+	LC_PROFILE_STUD_CYLINDER_COLOR,
+	LC_PROFILE_PART_EDGE_COLOR,
+	LC_PROFILE_BLACK_EDGE_COLOR,
+	LC_PROFILE_DARK_EDGE_COLOR,
+	LC_PROFILE_PART_EDGE_CONTRAST,
+	LC_PROFILE_PART_COLOR_VALUE_LD_INDEX,
+	LC_PROFILE_AUTOMATE_EDGE_COLOR,
+
 	LC_NUM_PROFILE_KEYS
 };
 
@@ -123,4 +150,3 @@ void lcSetProfileFloat(LC_PROFILE_KEY Key, float Value);
 void lcSetProfileString(LC_PROFILE_KEY Key, const QString& Value);
 void lcSetProfileStringList(LC_PROFILE_KEY Key, const QStringList& Value);
 void lcSetProfileBuffer(LC_PROFILE_KEY Key, const QByteArray& Buffer);
-
