@@ -4,10 +4,13 @@
 #include "piece.h"
 #include "pieceinf.h"
 #include "lc_mainwindow.h"
+#include "lc_viewwidget.h"
+#include "lc_previewwidget.h"
 
 lcTimelineWidget::lcTimelineWidget(QWidget* Parent)
 	: QTreeWidget(Parent)
 {
+	mCurrentStepItem = nullptr;
 	mIgnoreUpdates = false;
 
 	setSelectionMode(QAbstractItemView::ExtendedSelection);
@@ -19,6 +22,7 @@ lcTimelineWidget::lcTimelineWidget(QWidget* Parent)
 
 	invisibleRootItem()->setFlags(invisibleRootItem()->flags() & ~Qt::ItemIsDropEnabled);
 
+	connect(this, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), SLOT(CurrentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)));
 	connect(this, SIGNAL(itemSelectionChanged()), SLOT(ItemSelectionChanged()));
 	connect(this, SIGNAL(customContextMenuRequested(QPoint)), SLOT(CustomMenuRequested(QPoint)));
 }
@@ -47,7 +51,8 @@ void lcTimelineWidget::CustomMenuRequested(QPoint Pos)
 	}
 
 	Menu->addAction(gMainWindow->mActions[LC_TIMELINE_SET_CURRENT]);
-	Menu->addAction(gMainWindow->mActions[LC_TIMELINE_INSERT]);
+	Menu->addAction(gMainWindow->mActions[LC_TIMELINE_INSERT_BEFORE]);
+	Menu->addAction(gMainWindow->mActions[LC_TIMELINE_INSERT_AFTER]);
 	Menu->addAction(gMainWindow->mActions[LC_TIMELINE_DELETE]);
 	Menu->addAction(gMainWindow->mActions[LC_TIMELINE_MOVE_SELECTION]);
 
@@ -73,6 +78,7 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 
 	if (!Model)
 	{
+		mCurrentStepItem = nullptr;
 		mItems.clear();
 		clear();
 		blockSignals(Blocked);
@@ -81,11 +87,15 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 
 	if (Clear)
 	{
+		mCurrentStepItem = nullptr;
 		mItems.clear();
 		clear();
 	}
 
-	lcStep LastStep = lcMax(Model->GetLastStep(), Model->GetCurrentStep());
+	lcStep LastStep = Model->GetLastStep();
+	if (Model->HasPieces())
+		LastStep++;
+	LastStep  = lcMax(LastStep, Model->GetCurrentStep());
 
 	for (int TopLevelItemIdx = LastStep; TopLevelItemIdx < topLevelItemCount(); )
 	{
@@ -99,6 +109,9 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 			delete PieceItem;
 		}
 
+		if (mCurrentStepItem == StepItem)
+			mCurrentStepItem = nullptr;
+
 		delete StepItem;
 	}
 
@@ -153,7 +166,7 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 			{
 				PieceItem = new QTreeWidgetItem();
 				PieceItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled);
-				PieceItem->setData(0, Qt::UserRole, qVariantFromValue<uintptr_t>((uintptr_t)Piece));
+				PieceItem->setData(0, Qt::UserRole, QVariant::fromValue<uintptr_t>((uintptr_t)Piece));
 				StepItem->insertChild(PieceItemIndex, PieceItem);
 				mItems[Piece] = PieceItem;
 
@@ -177,7 +190,7 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 		{
 			PieceItem->setText(0, Piece->mPieceInfo->m_strDescription);
 
-			int ColorIndex = Piece->mColorIndex;
+			int ColorIndex = Piece->GetColorIndex();
 			if (!mIcons.contains(ColorIndex))
 			{
 				int Size = rowHeight(indexFromItem(PieceItem));
@@ -196,9 +209,13 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 			PieceItem->setIcon(0, mIcons[ColorIndex]);
 
 			QColor Color = palette().text().color();
-			if (Piece->IsHidden())
+
+			if (Piece->mPieceInfo->IsPlaceholder())
+				Color = QColor(208, 0, 0);
+			else if (Piece->IsHidden())
 				Color.setAlpha(128);
-			PieceItem->setTextColor(0, Color);
+
+			PieceItem->setForeground(0, Color);
 		}
 
 		PieceItem->setSelected(Piece->IsSelected());
@@ -226,10 +243,39 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 		StepItem = topLevelItem(Step - 1);
 		PieceItemIndex = 0;
 	}
+	
+	UpdateCurrentStepItem();
 
 	blockSignals(Blocked);
 }
 
+void lcTimelineWidget::UpdateCurrentStepItem()
+{
+	lcModel* Model = gMainWindow->GetActiveModel();
+	lcStep CurrentStep = Model->GetCurrentStep();
+	QTreeWidgetItem* CurrentStepItem = topLevelItem(CurrentStep - 1);
+
+	if (CurrentStepItem != mCurrentStepItem)
+	{
+		if (mCurrentStepItem)
+		{
+			QFont Font = mCurrentStepItem->font(0);
+			Font.setBold(false);
+			mCurrentStepItem->setFont(0, Font);
+		}
+
+		if (CurrentStepItem)
+		{
+			QFont Font = CurrentStepItem->font(0);
+			Font.setBold(true);
+			CurrentStepItem->setFont(0, Font);
+			setCurrentItem(CurrentStepItem);
+		}
+
+		mCurrentStepItem = CurrentStepItem;
+	}
+}
+
 void lcTimelineWidget::UpdateSelection()
 {
 	if (mIgnoreUpdates)
@@ -261,7 +307,7 @@ void lcTimelineWidget::UpdateSelection()
 	blockSignals(Blocked);
 }
 
-void lcTimelineWidget::InsertStep()
+void lcTimelineWidget::InsertStepBefore()
 {
 	QTreeWidgetItem* CurrentItem = currentItem();
 
@@ -279,6 +325,24 @@ void lcTimelineWidget::InsertStep()
 	gMainWindow->GetActiveModel()->InsertStep(Step + 1);
 }
 
+void lcTimelineWidget::InsertStepAfter()
+{
+	QTreeWidgetItem* CurrentItem = currentItem();
+
+	if (!CurrentItem)
+		return;
+
+	if (CurrentItem->parent())
+		CurrentItem = CurrentItem->parent();
+
+	int Step = indexOfTopLevelItem(CurrentItem);
+
+	if (Step == -1)
+		return;
+
+	gMainWindow->GetActiveModel()->InsertStep(Step + 2);
+}
+
 void lcTimelineWidget::RemoveStep()
 {
 	QTreeWidgetItem* CurrentItem = currentItem();
@@ -311,6 +375,7 @@ void lcTimelineWidget::MoveSelection()
 
 	if (Step == -1)
 		return;
+	Step++;
 
 	QList<QTreeWidgetItem*> SelectedItems = selectedItems();
 
@@ -326,6 +391,11 @@ void lcTimelineWidget::MoveSelection()
 	}
 
 	UpdateModel();
+
+	lcModel* Model = gMainWindow->GetActiveModel();
+
+	if (Step > static_cast<int>(Model->GetCurrentStep()))
+		Model->SetCurrentStep(Step);
 }
 
 void lcTimelineWidget::SetCurrentStep()
@@ -346,6 +416,14 @@ void lcTimelineWidget::SetCurrentStep()
 	gMainWindow->GetActiveModel()->SetCurrentStep(Step + 1);
 }
 
+void lcTimelineWidget::CurrentItemChanged(QTreeWidgetItem* Current, QTreeWidgetItem* Previous)
+{
+	Q_UNUSED(Previous);
+
+	if (Current && !Current->parent())
+		SetCurrentStep();
+}
+
 void lcTimelineWidget::ItemSelectionChanged()
 {
 	lcArray<lcObject*> Selection;
@@ -371,7 +449,10 @@ void lcTimelineWidget::ItemSelectionChanged()
 	mIgnoreUpdates = true;
 	lcModel* Model = gMainWindow->GetActiveModel();
 	if (LastStep > Model->GetCurrentStep())
+	{
 		Model->SetCurrentStep(LastStep);
+		UpdateCurrentStepItem();
+	}
 	Model->SetSelectionAndFocus(Selection, CurrentPiece, LC_PIECE_SECTION_POSITION, false);
 	mIgnoreUpdates = false;
 	blockSignals(Blocked);
@@ -379,8 +460,41 @@ void lcTimelineWidget::ItemSelectionChanged()
 
 void lcTimelineWidget::dropEvent(QDropEvent* Event)
 {
+	QTreeWidgetItem* DropItem = itemAt(Event->pos());
+	lcModel* Model = gMainWindow->GetActiveModel();
+
+	if (DropItem)
+	{
+		QTreeWidgetItem* ParentItem = DropItem->parent();
+		lcStep Step = indexOfTopLevelItem(ParentItem ? ParentItem : DropItem) + 1;
+
+		if (Step > Model->GetCurrentStep())
+			Model->SetCurrentStep(Step);
+	}
+
+	QList<QTreeWidgetItem*> SelectedItems = selectedItems();
+	clearSelection();
+
+	auto SortItems = [this](QTreeWidgetItem* Item1, QTreeWidgetItem* Item2)
+	{
+		QTreeWidgetItem* StepItem1 = Item1->parent();
+		QTreeWidgetItem* StepItem2 = Item2->parent();
+
+		if (StepItem1 == StepItem2)
+			return StepItem1->indexOfChild(Item1) < StepItem1->indexOfChild(Item2);
+
+		return indexOfTopLevelItem(StepItem1) < indexOfTopLevelItem(StepItem2);
+	};
+
+	std::sort(SelectedItems.begin(), SelectedItems.end(), SortItems);
+
+	for (QTreeWidgetItem* SelectedItem : SelectedItems)
+		SelectedItem->setSelected(true);
+
 	QTreeWidget::dropEvent(Event);
+
 	UpdateModel();
+	Update(false, false);
 }
 
 void lcTimelineWidget::mousePressEvent(QMouseEvent* Event)
@@ -399,6 +513,35 @@ void lcTimelineWidget::mousePressEvent(QMouseEvent* Event)
 		QTreeWidget::mousePressEvent(Event);
 }
 
+void lcTimelineWidget::mouseDoubleClickEvent(QMouseEvent* MouseEvent)
+{
+	if (MouseEvent->button() == Qt::LeftButton)
+	{
+		QTreeWidgetItem* CurrentItem = currentItem();
+		PreviewSelection(CurrentItem);
+	}
+
+	QTreeWidget::mouseDoubleClickEvent(MouseEvent);
+}
+
+void lcTimelineWidget::PreviewSelection(QTreeWidgetItem* CurrentItem)
+{
+	if (!CurrentItem)
+		return;
+
+	lcPiece* Piece = (lcPiece*)CurrentItem->data(0, Qt::UserRole).value<uintptr_t>();
+
+	if (!Piece)
+		return;
+
+	PieceInfo* Info = Piece->mPieceInfo;
+
+	if (!Info)
+		return;
+
+	gMainWindow->PreviewPiece(Info->mFileName, Piece->GetColorCode(), false);
+}
+
 void lcTimelineWidget::UpdateModel()
 {
 	QList<QPair<lcPiece*, lcStep>> PieceSteps;
