@@ -18,7 +18,7 @@
 #define LC_PIECE_CONTROL_POINT_SIZE 10.0f
 
 lcPiece::lcPiece(PieceInfo* Info)
-	: lcObject(LC_OBJECT_PIECE)
+	: lcObject(lcObjectType::Piece)
 {
 	mMesh = nullptr;
 	SetPieceInfo(Info, QString(), true);
@@ -33,7 +33,7 @@ lcPiece::lcPiece(PieceInfo* Info)
 }
 
 lcPiece::lcPiece(const lcPiece& Other)
-	: lcObject(LC_OBJECT_PIECE)
+	: lcObject(lcObjectType::Piece)
 {
 	mMesh = nullptr;
 	SetPieceInfo(Other.mPieceInfo, Other.mID, true);
@@ -44,7 +44,9 @@ lcPiece::lcPiece(const lcPiece& Other)
 	mStepHide = Other.mStepHide;
 	mGroup = Other.mGroup;
 	mFileLine = -1;
+
 	mPivotMatrix = Other.mPivotMatrix;
+	mState |= ( Other.mState & LC_PIECE_PIVOT_POINT_VALID );
 
 	mPositionKeys = Other.mPositionKeys;
 	mRotationKeys = Other.mRotationKeys;
@@ -121,10 +123,10 @@ void lcPiece::SaveLDraw(QTextStream& Stream) const
 	}
 
 	if (mPositionKeys.GetSize() > 1)
-		SaveKeysLDraw(Stream, mPositionKeys, "PIECE POSITION_KEY ");
+		mPositionKeys.SaveKeysLDraw(Stream, "PIECE POSITION_KEY ");
 
 	if (mRotationKeys.GetSize() > 1)
-		SaveKeysLDraw(Stream, mRotationKeys, "PIECE ROTATION_KEY ");
+		mRotationKeys.SaveKeysLDraw(Stream, "PIECE ROTATION_KEY ");
 
 	Stream << "1 " << mColorCode << ' ';
 
@@ -161,9 +163,9 @@ bool lcPiece::ParseLDrawLine(QTextStream& Stream)
 			mState |= LC_PIECE_PIVOT_POINT_VALID;
 		}
 		else if (Token == QLatin1String("POSITION_KEY"))
-			LoadKeysLDraw(Stream, mPositionKeys);
+			mPositionKeys.LoadKeysLDraw(Stream);
 		else if (Token == QLatin1String("ROTATION_KEY"))
-			LoadKeysLDraw(Stream, mRotationKeys);
+			mRotationKeys.LoadKeysLDraw(Stream);
 	}
 
 	return false;
@@ -178,6 +180,8 @@ bool lcPiece::FileLoad(lcFile& file)
 	if (version > 12) // LeoCAD 0.80
 		return false;
 
+	const float PositionScale = (version < 12) ? 25.0f : 1.0f;
+
 	if (version > 8)
 	{
 		if (file.ReadU8() != 1)
@@ -196,9 +200,9 @@ bool lcPiece::FileLoad(lcFile& file)
 			file.ReadU8(&type, 1);
 
 			if (type == 0)
-				ChangeKey(mPositionKeys, lcVector3(param[0], param[1], param[2]), time, true);
+				mPositionKeys.ChangeKey(lcVector3(param[0], param[1], param[2]) * PositionScale, time, true);
 			else if (type == 1)
-				ChangeKey(mRotationKeys, lcMatrix33FromAxisAngle(lcVector3(param[0], param[1], param[2]), param[3] * LC_DTOR), time, true);
+				mRotationKeys.ChangeKey(lcMatrix33FromAxisAngle(lcVector3(param[0], param[1], param[2]), param[3] * LC_DTOR), time, true);
 		}
 
 		file.ReadU32(&n, 1);
@@ -210,50 +214,71 @@ bool lcPiece::FileLoad(lcFile& file)
 		}
 	}
 
-  if (version < 9)
-  {
-    quint16 time;
-    quint8 type;
+	if (version < 9)
+	{
+		quint16 time;
+		quint8 type;
 
-    if (version > 5)
-    {
-      quint32 keys;
-      float param[4];
+		if (version > 5)
+		{
+			quint32 keys;
+			float param[4];
 
-      file.ReadU32(&keys, 1);
-      while (keys--)
-      {
-        file.ReadFloats(param, 4);
-        file.ReadU16(&time, 1);
-        file.ReadU8(&type, 1);
-
-		if (type == 0)
-			ChangeKey(mPositionKeys, lcVector3(param[0], param[1], param[2]), time, true);
-		else if (type == 1)
-			ChangeKey(mRotationKeys, lcMatrix33FromAxisAngle(lcVector3(param[0], param[1], param[2]), param[3] * LC_DTOR), time, true);
-      }
-
-      file.ReadU32(&keys, 1);
-      while (keys--)
-      {
-        file.ReadFloats(param, 4);
-        file.ReadU16(&time, 1);
-        file.ReadU8(&type, 1);
-      }
-    }
-    else
-    {
-      if (version > 2)
-      {
-        file.ReadU8(&ch, 1);
-
-        while (ch--)
-        {
-			lcMatrix44 ModelWorld;
-
-			if (version > 3)
+			file.ReadU32(&keys, 1);
+			while (keys--)
 			{
-				file.ReadFloats(ModelWorld, 16);
+				file.ReadFloats(param, 4);
+				file.ReadU16(&time, 1);
+				file.ReadU8(&type, 1);
+
+				if (type == 0)
+					mPositionKeys.ChangeKey(lcVector3(param[0], param[1], param[2]) * PositionScale, time, true);
+				else if (type == 1)
+					mRotationKeys.ChangeKey(lcMatrix33FromAxisAngle(lcVector3(param[0], param[1], param[2]), param[3] * LC_DTOR), time, true);
+			}
+
+			file.ReadU32(&keys, 1);
+			while (keys--)
+			{
+				file.ReadFloats(param, 4);
+				file.ReadU16(&time, 1);
+				file.ReadU8(&type, 1);
+			}
+		}
+		else
+		{
+			if (version > 2)
+			{
+				file.ReadU8(&ch, 1);
+
+				while (ch--)
+				{
+					lcMatrix44 ModelWorld;
+
+					if (version > 3)
+					{
+						file.ReadFloats(ModelWorld, 16);
+					}
+					else
+					{
+						lcVector3 Translation;
+						float Rotation[3];
+						file.ReadFloats(Translation, 3);
+						file.ReadFloats(Rotation, 3);
+						ModelWorld = lcMatrix44Translation(Translation);
+						ModelWorld = lcMul(lcMatrix44RotationZ(Rotation[2] * LC_DTOR), lcMul(lcMatrix44RotationY(Rotation[1] * LC_DTOR), lcMul(lcMatrix44RotationX(Rotation[0] * LC_DTOR), ModelWorld)));
+					}
+
+					quint8 b;
+					file.ReadU8(&b, 1);
+					time = b;
+
+					mPositionKeys.ChangeKey(ModelWorld.GetTranslation() * PositionScale, 1, true);
+					mRotationKeys.ChangeKey(lcMatrix33(ModelWorld), time, true);
+
+					qint32 bl;
+					file.ReadS32(&bl, 1);
+				}
 			}
 			else
 			{
@@ -261,46 +286,25 @@ bool lcPiece::FileLoad(lcFile& file)
 				float Rotation[3];
 				file.ReadFloats(Translation, 3);
 				file.ReadFloats(Rotation, 3);
-				ModelWorld = lcMatrix44Translation(Translation);
+				lcMatrix44 ModelWorld = lcMatrix44Translation(Translation);
 				ModelWorld = lcMul(lcMatrix44RotationZ(Rotation[2] * LC_DTOR), lcMul(lcMatrix44RotationY(Rotation[1] * LC_DTOR), lcMul(lcMatrix44RotationX(Rotation[0] * LC_DTOR), ModelWorld)));
+
+				mPositionKeys.ChangeKey(lcVector3(ModelWorld.r[3][0], ModelWorld.r[3][1], ModelWorld.r[3][2]) * PositionScale, 1, true);
+				mRotationKeys.ChangeKey(lcMatrix33(ModelWorld), 1, true);
 			}
+		}
+	}
 
-			quint8 b;
-			file.ReadU8(&b, 1);
-			time = b;
-
-			ChangeKey(mPositionKeys, ModelWorld.GetTranslation(), 1, true);
-			ChangeKey(mRotationKeys, lcMatrix33(ModelWorld), time, true);
-
-			qint32 bl;
-			file.ReadS32(&bl, 1);
-        }
-      }
-      else
-      {
-			lcVector3 Translation;
-			float Rotation[3];
-			file.ReadFloats(Translation, 3);
-			file.ReadFloats(Rotation, 3);
-			lcMatrix44 ModelWorld = lcMatrix44Translation(Translation);
-			ModelWorld = lcMul(lcMatrix44RotationZ(Rotation[2] * LC_DTOR), lcMul(lcMatrix44RotationY(Rotation[1] * LC_DTOR), lcMul(lcMatrix44RotationX(Rotation[0] * LC_DTOR), ModelWorld)));
-
-			ChangeKey(mPositionKeys, lcVector3(ModelWorld.r[3][0], ModelWorld.r[3][1], ModelWorld.r[3][2]), 1, true);
-			ChangeKey(mRotationKeys, lcMatrix33(ModelWorld), 1, true);
-	  }
-    }
-  }
-
-  // Common to all versions.
+	// Common to all versions.
 	char name[LC_PIECE_NAME_LEN];
-  if (version < 10)
-  {
-	  memset(name, 0, LC_PIECE_NAME_LEN);
-	  file.ReadBuffer(name, 9);
-  }
-  else
-	  file.ReadBuffer(name, LC_PIECE_NAME_LEN);
-  strcat(name, ".dat");
+	if (version < 10)
+	{
+		memset(name, 0, LC_PIECE_NAME_LEN);
+		file.ReadBuffer(name, 9);
+	}
+	else
+		file.ReadBuffer(name, LC_PIECE_NAME_LEN);
+	strcat(name, ".dat");
 
 	PieceInfo* pInfo = lcGetPiecesLibrary()->FindPiece(name, nullptr, true, false);
 	SetPieceInfo(pInfo, QString(), true);
@@ -321,63 +325,57 @@ bool lcPiece::FileLoad(lcFile& file)
 		file.ReadU32(&mColorCode, 1);
 	mColorIndex = lcGetColorIndex(mColorCode);
 
-  quint8 Step;
-  file.ReadU8(&Step, 1);
-  mStepShow = Step;
-  if (version > 1)
-  {
-    file.ReadU8(&Step, 1);
-	mStepHide = Step == 255 ? LC_STEP_MAX : Step;
-  }
-  else
-    mStepHide = LC_STEP_MAX;
-
-  if (version > 5)
-  {
-	file.ReadU16(); // m_nFrameShow
-	file.ReadU16(); // m_nFrameHide
-
-    if (version > 7)
-    {
-      quint8 Hidden;
-      file.ReadU8(&Hidden, 1);
-	  if (Hidden & 1)
-		  mState |= LC_PIECE_HIDDEN;
-      file.ReadU8(&ch, 1);
-	  file.Seek(ch, SEEK_CUR);
-    }
-    else
-    {
-      qint32 hide;
-      file.ReadS32(&hide, 1);
-      if (hide != 0)
-        mState |= LC_PIECE_HIDDEN;
-	  file.Seek(81, SEEK_CUR);
-    }
-
-    // 7 (0.64)
-    qint32 i = -1;
-    if (version > 6)
-      file.ReadS32(&i, 1);
-    mGroup = (lcGroup*)(quintptr)i;
-  }
-  else
-  {
-    file.ReadU8(&ch, 1);
-    if (ch == 0)
-      mGroup = (lcGroup*)-1;
-    else
-      mGroup = (lcGroup*)(quintptr)ch;
-
-    file.ReadU8(&ch, 1);
-    if (ch & 0x01)
-      mState |= LC_PIECE_HIDDEN;
-  }
-
-	if (version < 12)
+	quint8 Step;
+	file.ReadU8(&Step, 1);
+	mStepShow = Step;
+	if (version > 1)
 	{
-		for (int KeyIdx = 0; KeyIdx < mPositionKeys.GetSize(); KeyIdx++)
-			mPositionKeys[KeyIdx].Value *= 25.0f;
+		file.ReadU8(&Step, 1);
+		mStepHide = Step == 255 ? LC_STEP_MAX : Step;
+	}
+	else
+		mStepHide = LC_STEP_MAX;
+
+	if (version > 5)
+	{
+		file.ReadU16(); // m_nFrameShow
+		file.ReadU16(); // m_nFrameHide
+
+		if (version > 7)
+		{
+			quint8 Hidden;
+			file.ReadU8(&Hidden, 1);
+			if (Hidden & 1)
+				mState |= LC_PIECE_HIDDEN;
+			file.ReadU8(&ch, 1);
+			file.Seek(ch, SEEK_CUR);
+		}
+		else
+		{
+			qint32 hide;
+			file.ReadS32(&hide, 1);
+			if (hide != 0)
+				mState |= LC_PIECE_HIDDEN;
+			file.Seek(81, SEEK_CUR);
+		}
+
+		// 7 (0.64)
+		qint32 i = -1;
+		if (version > 6)
+			file.ReadS32(&i, 1);
+		mGroup = (lcGroup*)(quintptr)i;
+	}
+	else
+	{
+		file.ReadU8(&ch, 1);
+		if (ch == 0)
+			mGroup = (lcGroup*)-1;
+		else
+			mGroup = (lcGroup*)(quintptr)ch;
+
+		file.ReadU8(&ch, 1);
+		if (ch & 0x01)
+			mState |= LC_PIECE_HIDDEN;
 	}
 
 	return true;
@@ -388,9 +386,9 @@ void lcPiece::Initialize(const lcMatrix44& WorldMatrix, lcStep Step)
 	mStepShow = Step;
 
 	if (mPositionKeys.IsEmpty())
-		ChangeKey(mPositionKeys, WorldMatrix.GetTranslation(), 1, true);
+		mPositionKeys.ChangeKey(WorldMatrix.GetTranslation(), 1, true);
 	if (mRotationKeys.IsEmpty())
-		ChangeKey(mRotationKeys, lcMatrix33(WorldMatrix), 1, true);
+		mRotationKeys.ChangeKey(lcMatrix33(WorldMatrix), 1, true);
 
 	UpdatePosition(Step);
 }
@@ -424,8 +422,8 @@ void lcPiece::InsertTime(lcStep Start, lcStep Time)
 		}
 	}
 
-	lcObject::InsertTime(mPositionKeys, Start, Time);
-	lcObject::InsertTime(mRotationKeys, Start, Time);
+	mPositionKeys.InsertTime(Start, Time);
+	mRotationKeys.InsertTime(Start, Time);
 }
 
 void lcPiece::RemoveTime(lcStep Start, lcStep Time)
@@ -457,8 +455,8 @@ void lcPiece::RemoveTime(lcStep Start, lcStep Time)
 		}
 	}
 
-	lcObject::RemoveTime(mPositionKeys, Start, Time);
-	lcObject::RemoveTime(mRotationKeys, Start, Time);
+	mPositionKeys.RemoveTime(Start, Time);
+	mRotationKeys.RemoveTime(Start, Time);
 }
 
 void lcPiece::RayTest(lcObjectRayTest& ObjectRayTest) const
@@ -519,7 +517,7 @@ void lcPiece::DrawInterface(lcContext* Context, const lcScene& Scene) const
 	const lcVector3& Max = BoundingBox.Max;
 	lcVector3 Edge((Max - Min) * 0.33f);
 
-	float LineVerts[48][3] =
+	const float LineVerts[48][3] =
 	{
 		{ Max[0], Max[1], Max[2] }, { Max[0] - Edge[0], Max[1], Max[2] },
 		{ Max[0], Max[1], Max[2] }, { Max[0], Max[1] - Edge[1], Max[2] },
@@ -555,7 +553,7 @@ void lcPiece::DrawInterface(lcContext* Context, const lcScene& Scene) const
 	};
 
 	lcMatrix44 WorldMatrix = Scene.ApplyActiveSubmodelTransform(mModelWorld);
-	Context->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
+	Context->SetMaterial(lcMaterialType::UnlitColor);
 	Context->SetWorldMatrix(WorldMatrix);
 
 	if (IsFocused(LC_PIECE_SECTION_POSITION))
@@ -591,7 +589,7 @@ void lcPiece::DrawInterface(lcContext* Context, const lcScene& Scene) const
 		Context->DrawIndexedPrimitives(GL_LINES, 24, GL_UNSIGNED_SHORT, 0);
 	}
 
-	if (AreControlPointsVisible())
+	if (!mControlPoints.IsEmpty() && AreControlPointsVisible())
 	{
 		float Verts[8 * 3];
 		float* CurVert = Verts;
@@ -615,7 +613,7 @@ void lcPiece::DrawInterface(lcContext* Context, const lcScene& Scene) const
 		};
 
 		glEnable(GL_BLEND);
-		glEnable(GL_CULL_FACE);
+		Context->EnableCullFace(true);
 
 		for (int ControlPointIdx = 0; ControlPointIdx < mControlPoints.GetSize(); ControlPointIdx++)
 		{
@@ -633,7 +631,7 @@ void lcPiece::DrawInterface(lcContext* Context, const lcScene& Scene) const
 			Context->DrawIndexedPrimitives(GL_TRIANGLES, 36, GL_UNSIGNED_SHORT, 0);
 		}
 
-		glDisable(GL_CULL_FACE);
+		Context->EnableCullFace(false);
 		glDisable(GL_BLEND);
 	}
 }
@@ -641,61 +639,85 @@ void lcPiece::DrawInterface(lcContext* Context, const lcScene& Scene) const
 void lcPiece::RemoveKeyFrames()
 {
 	mPositionKeys.RemoveAll();
-	ChangeKey(mPositionKeys, mModelWorld.GetTranslation(), 1, true);
+	mPositionKeys.ChangeKey(mModelWorld.GetTranslation(), 1, true);
 
 	mRotationKeys.RemoveAll();
-	ChangeKey(mRotationKeys, lcMatrix33(mModelWorld), 1, true);
+	mRotationKeys.ChangeKey(lcMatrix33(mModelWorld), 1, true);
 }
 
-void lcPiece::AddMainModelRenderMeshes(lcScene& Scene, bool Highlight) const
+void lcPiece::AddMainModelRenderMeshes(lcScene* Scene, bool Highlight, bool Fade) const
 {
-	lcRenderMeshState RenderMeshState = lcRenderMeshState::NORMAL;
+	lcRenderMeshState RenderMeshState = lcRenderMeshState::Default;
 	bool ParentActive = false;
 
-	if (Scene.GetDrawInterface())
+	if (Highlight)
+		RenderMeshState = lcRenderMeshState::Highlighted;
+
+	if (Fade)
+		RenderMeshState = lcRenderMeshState::Faded;
+
+	if (Scene->GetDrawInterface())
 	{
-		lcPiece* ActiveSubmodelInstance = Scene.GetActiveSubmodelInstance();
+		lcPiece* ActiveSubmodelInstance = Scene->GetActiveSubmodelInstance();
 
 		if (!ActiveSubmodelInstance)
-			RenderMeshState = IsFocused() ? lcRenderMeshState::FOCUSED : (IsSelected() ? lcRenderMeshState::SELECTED : lcRenderMeshState::NORMAL);
+			RenderMeshState = IsFocused() ? lcRenderMeshState::Focused : (IsSelected() ? lcRenderMeshState::Selected : RenderMeshState);
 		else if (ActiveSubmodelInstance == this)
 			ParentActive = true;
 		else
-			RenderMeshState = lcRenderMeshState::DISABLED;
+			RenderMeshState = lcRenderMeshState::Faded;
 	}
-	else if (Highlight)
-		RenderMeshState = lcRenderMeshState::HIGHLIGHT;
 
 	if (!mMesh)
 		mPieceInfo->AddRenderMeshes(Scene, mModelWorld, mColorIndex, RenderMeshState, ParentActive);
 	else
-		Scene.AddMesh(mMesh, mModelWorld, mColorIndex, RenderMeshState, mPieceInfo->mFlags);
+		Scene->AddMesh(mMesh, mModelWorld, mColorIndex, RenderMeshState);
 
-	if (RenderMeshState == lcRenderMeshState::FOCUSED || RenderMeshState == lcRenderMeshState::SELECTED)
-		Scene.AddInterfaceObject(this);
+	if (RenderMeshState == lcRenderMeshState::Focused || RenderMeshState == lcRenderMeshState::Selected)
+		Scene->AddInterfaceObject(this);
 }
 
-void lcPiece::AddSubModelRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMatrix, int DefaultColorIndex, lcRenderMeshState RenderMeshState, bool ParentActive) const
+void lcPiece::AddSubModelRenderMeshes(lcScene* Scene, const lcMatrix44& WorldMatrix, int DefaultColorIndex, lcRenderMeshState RenderMeshState, bool ParentActive) const
 {
 	int ColorIndex = mColorIndex;
 
 	if (ColorIndex == gDefaultColor)
 		ColorIndex = DefaultColorIndex;
 
-	lcPiece* ActiveSubmodelInstance = Scene.GetActiveSubmodelInstance();
+	lcPiece* ActiveSubmodelInstance = Scene->GetActiveSubmodelInstance();
 
 	if (ActiveSubmodelInstance == this)
-		RenderMeshState = lcRenderMeshState::NORMAL;
+		RenderMeshState = lcRenderMeshState::Default;
 	else if (ParentActive)
-		RenderMeshState = IsFocused() ? lcRenderMeshState::FOCUSED : (IsSelected() ? lcRenderMeshState::SELECTED : lcRenderMeshState::NORMAL);
+		RenderMeshState = IsFocused() ? lcRenderMeshState::Focused : (IsSelected() ? lcRenderMeshState::Selected : lcRenderMeshState::Default);
 
 	if (!mMesh)
 		mPieceInfo->AddRenderMeshes(Scene, lcMul(mModelWorld, WorldMatrix), ColorIndex, RenderMeshState, ActiveSubmodelInstance == this);
 	else
-		Scene.AddMesh(mMesh, lcMul(mModelWorld, WorldMatrix), ColorIndex, RenderMeshState, mPieceInfo->mFlags);
+		Scene->AddMesh(mMesh, lcMul(mModelWorld, WorldMatrix), ColorIndex, RenderMeshState);
 
-	if (ParentActive && (RenderMeshState == lcRenderMeshState::FOCUSED || RenderMeshState == lcRenderMeshState::SELECTED))
-		Scene.AddInterfaceObject(this);
+	if (ParentActive && (RenderMeshState == lcRenderMeshState::Focused || RenderMeshState == lcRenderMeshState::Selected))
+		Scene->AddInterfaceObject(this);
+}
+
+void lcPiece::SubModelCompareBoundingBox(const lcMatrix44& WorldMatrix, lcVector3& Min, lcVector3& Max) const
+{
+	mPieceInfo->CompareBoundingBox(lcMul(mModelWorld, WorldMatrix), Min, Max);
+}
+
+void lcPiece::SubModelAddBoundingBoxPoints(const lcMatrix44& WorldMatrix, std::vector<lcVector3>& Points) const
+{
+	if (!mMesh)
+		mPieceInfo->AddSubModelBoundingBoxPoints(lcMul(mModelWorld, WorldMatrix), Points);
+	else
+	{
+		lcVector3 BoxPoints[8];
+
+		lcGetBoxCorners(mMesh->mBoundingBox, BoxPoints);
+
+		for (int i = 0; i < 8; i++)
+			Points.emplace_back(lcMul31(BoxPoints[i], mModelWorld));
+	}
 }
 
 void lcPiece::MoveSelected(lcStep Step, bool AddKey, const lcVector3& Distance)
@@ -792,32 +814,56 @@ void lcPiece::RotatePivotPoint(const lcMatrix33& RotationMatrix)
 
 quint32 lcPiece::GetAllowedTransforms() const
 {
+	const quint32 Move = LC_OBJECT_TRANSFORM_MOVE_X | LC_OBJECT_TRANSFORM_MOVE_Y | LC_OBJECT_TRANSFORM_MOVE_Z;
+	const quint32 Rotate = LC_OBJECT_TRANSFORM_ROTATE_X | LC_OBJECT_TRANSFORM_ROTATE_Y | LC_OBJECT_TRANSFORM_ROTATE_Z;
 	quint32 Section = GetFocusSection();
 
 	if (Section == LC_PIECE_SECTION_POSITION || Section == LC_PIECE_SECTION_INVALID)
-		return LC_OBJECT_TRANSFORM_MOVE_X | LC_OBJECT_TRANSFORM_MOVE_Y | LC_OBJECT_TRANSFORM_MOVE_Z | LC_OBJECT_TRANSFORM_ROTATE_X | LC_OBJECT_TRANSFORM_ROTATE_Y | LC_OBJECT_TRANSFORM_ROTATE_Z;
+		return Move | Rotate;
 
 	lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
-	if (!SynthInfo)
-		return 0;
+	if (SynthInfo)
+	{
+		if (SynthInfo->IsUnidirectional())
+			return LC_OBJECT_TRANSFORM_MOVE_Z;
 
-	if (SynthInfo->IsCurve())
-		return LC_OBJECT_TRANSFORM_MOVE_X | LC_OBJECT_TRANSFORM_MOVE_Y | LC_OBJECT_TRANSFORM_MOVE_Z | LC_OBJECT_TRANSFORM_ROTATE_X | LC_OBJECT_TRANSFORM_ROTATE_Y | LC_OBJECT_TRANSFORM_ROTATE_Z | LC_OBJECT_TRANSFORM_SCALE_X;
-	else
-		return LC_OBJECT_TRANSFORM_MOVE_Z;
+		if (SynthInfo->IsCurve())
+			return Move | Rotate | LC_OBJECT_TRANSFORM_SCALE_X;
+
+		if (SynthInfo->IsNondirectional())
+			return Move;
+	}
+
+	return 0;
+}
+
+bool lcPiece::CanAddControlPoint() const
+{
+	if (mControlPoints.GetSize() >= LC_MAX_CONTROL_POINTS)
+		return false;
+
+	lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
+	return SynthInfo && SynthInfo->CanAddControlPoints();
+}
+
+bool lcPiece::CanRemoveControlPoint() const
+{
+	quint32 Section = GetFocusSection();
+	return Section >= LC_PIECE_SECTION_CONTROL_POINT_FIRST && Section <= LC_PIECE_SECTION_CONTROL_POINT_LAST && mControlPoints.GetSize() > 2;
 }
 
 bool lcPiece::InsertControlPoint(const lcVector3& WorldStart, const lcVector3& WorldEnd)
 {
-	lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
-	if (!SynthInfo || !SynthInfo->CanAddControlPoints())
+	if (!CanAddControlPoint())
 		return false;
 
 	lcMatrix44 InverseWorldMatrix = lcMatrix44AffineInverse(mModelWorld);
 	lcVector3 Start = lcMul31(WorldStart, InverseWorldMatrix);
 	lcVector3 End = lcMul31(WorldEnd, InverseWorldMatrix);
 
+	lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
 	int ControlPointIndex = SynthInfo->InsertControlPoint(mControlPoints, Start, End);
+
 	if (ControlPointIndex)
 	{
 		SetFocused(GetFocusSection(), false);
@@ -845,9 +891,25 @@ bool lcPiece::RemoveFocusedControlPoint()
 	return true;
 }
 
-const char* lcPiece::GetName() const
+void lcPiece::VerifyControlPoints(lcArray<lcPieceControlPoint>& ControlPoints) const
 {
-	return mPieceInfo->m_strDescription;
+	lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
+	if (!SynthInfo)
+	{
+		ControlPoints.RemoveAll();
+	}
+	else
+	{
+		if (ControlPoints.GetSize() > LC_MAX_CONTROL_POINTS)
+			ControlPoints.SetSize(LC_MAX_CONTROL_POINTS);
+
+		SynthInfo->VerifyControlPoints(ControlPoints);
+	}
+}
+
+QString lcPiece::GetName() const
+{
+	return QString::fromLatin1(mPieceInfo->m_strDescription);
 }
 
 bool lcPiece::IsVisible(lcStep Step) const
@@ -889,19 +951,21 @@ const lcBoundingBox& lcPiece::GetBoundingBox() const
 
 void lcPiece::CompareBoundingBox(lcVector3& Min, lcVector3& Max) const
 {
-	lcVector3 Points[8];
-
 	if (!mMesh)
-		lcGetBoxCorners(GetBoundingBox(), Points);
+		mPieceInfo->CompareBoundingBox(mModelWorld, Min, Max);
 	else
+	{
+		lcVector3 Points[8];
+
 		lcGetBoxCorners(mMesh->mBoundingBox, Points);
 
-	for (int i = 0; i < 8; i++)
-	{
-		lcVector3 Point = lcMul31(Points[i], mModelWorld);
+		for (int i = 0; i < 8; i++)
+		{
+			lcVector3 Point = lcMul31(Points[i], mModelWorld);
 
-		Min = lcMin(Point, Min);
-		Max = lcMax(Point, Max);
+			Min = lcMin(Point, Min);
+			Max = lcMax(Point, Max);
+		}
 	}
 }
 
@@ -912,8 +976,8 @@ lcGroup* lcPiece::GetTopGroup()
 
 void lcPiece::UpdatePosition(lcStep Step)
 {
-	lcVector3 Position = CalculateKey(mPositionKeys, Step);
-	lcMatrix33 Rotation = CalculateKey(mRotationKeys, Step);
+	lcVector3 Position = mPositionKeys.CalculateKey(Step);
+	lcMatrix33 Rotation = mRotationKeys.CalculateKey(Step);
 
 	mModelWorld = lcMatrix44(Rotation, Position);
 }
