@@ -4,14 +4,13 @@
 #include "lc_math.h"
 #include "lc_array.h"
 
-#define LC_PIECE_HAS_DEFAULT        0x01 // Piece has triangles using the default color
-#define LC_PIECE_HAS_SOLID          0x02 // Piece has triangles using a solid color
-#define LC_PIECE_HAS_TRANSLUCENT    0x04 // Piece has triangles using a translucent color
-#define LC_PIECE_HAS_LINES          0x08 // Piece has lines
-#define LC_PIECE_PLACEHOLDER        0x10 // Placeholder for a piece not in the library
-#define LC_PIECE_MODEL              0x20 // Piece is a model
-#define LC_PIECE_PROJECT            0x40 // Piece is a project
-#define LC_PIECE_HAS_TEXTURE        0x80 // Piece has sections using textures
+enum class lcPieceInfoType
+{
+	Part,
+	Placeholder,
+	Model,
+	Project
+};
 
 #define LC_PIECE_NAME_LEN 256
 
@@ -31,6 +30,7 @@ struct lcModelPartsEntry
 };
 
 class lcSynthInfo;
+enum class lcZipFileType;
 
 class PieceInfo
 {
@@ -38,6 +38,11 @@ public:
 	PieceInfo();
 	~PieceInfo();
 
+	PieceInfo(const PieceInfo&) = delete;
+	PieceInfo(PieceInfo&&) = delete;
+	PieceInfo& operator=(const PieceInfo&) = delete;
+	PieceInfo& operator=(PieceInfo&&) = delete;
+
 	const lcBoundingBox& GetBoundingBox() const
 	{
 		return mBoundingBox;
@@ -95,25 +100,25 @@ public:
 
 	bool IsPlaceholder() const
 	{
-		return (mFlags & LC_PIECE_PLACEHOLDER) != 0;
+		return mType == lcPieceInfoType::Placeholder;
 	}
 
 	bool IsModel() const
 	{
-		return (mFlags & LC_PIECE_MODEL) != 0;
+		return mType == lcPieceInfoType::Model;
 	}
 
 	bool IsProject() const
 	{
-		return (mFlags & LC_PIECE_PROJECT) != 0;
+		return mType == lcPieceInfoType::Project;
 	}
 
 	bool IsTemporary() const
 	{
-		return (mFlags & (LC_PIECE_PLACEHOLDER | LC_PIECE_MODEL | LC_PIECE_PROJECT)) != 0;
+		return mType != lcPieceInfoType::Part;
 	}
 
-	void SetZipFile(int ZipFileType, int ZipFileIndex)
+	void SetZipFile(lcZipFileType ZipFileType, int ZipFileIndex)
 	{
 		mZipFileType = ZipFileType;
 		mZipFileIndex = ZipFileIndex;
@@ -121,8 +126,16 @@ public:
 
 	bool IsPatterned() const
 	{
+		if (mType != lcPieceInfoType::Part)
+			return false;
+
 		const char* Name = mFileName;
 
+		// Heuristic: Names matching '^[Uu]?[0-9]*[A-Za-z][^.][^.]' are patterned.
+
+		if (*Name == 'U' || *Name == 'u')
+			Name++;
+
 		while (*Name)
 		{
 			if (*Name < '0' || *Name > '9')
@@ -131,7 +144,10 @@ public:
 			Name++;
 		}
 
-		if (*Name == 'P' || *Name == 'p')
+		if (!*Name || !((*Name >= 'A' && *Name <= 'Z') || (*Name >= 'a' && *Name <= 'z')))
+			return false;
+
+		if (Name[1] && Name[1] != '.' && Name[2] && Name[2] != '.')
 			return true;
 
 		return false;
@@ -144,7 +160,7 @@ public:
 
 	void ZoomExtents(float FoV, float AspectRatio, lcMatrix44& ProjectionMatrix, lcMatrix44& ViewMatrix) const;
 	void AddRenderMesh(lcScene& Scene);
-	void AddRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMatrix, int ColorIndex, lcRenderMeshState RenderMeshState, bool ParentActive) const;
+	void AddRenderMeshes(lcScene* Scene, const lcMatrix44& WorldMatrix, int ColorIndex, lcRenderMeshState RenderMeshState, bool ParentActive) const;
 
 	void CreatePlaceholder(const char* Name);
 
@@ -155,8 +171,10 @@ public:
 	bool IncludesModel(const lcModel* Model) const;
 	bool MinIntersectDist(const lcVector3& Start, const lcVector3& End, float& MinDistance) const;
 	bool BoxTest(const lcMatrix44& WorldMatrix, const lcVector4 Planes[6]) const;
-	void GetPartsList(int DefaultColorIndex, bool IncludeSubmodels, lcPartsList& PartsList) const;
+	void GetPartsList(int DefaultColorIndex, bool ScanSubModels, bool AddSubModels, lcPartsList& PartsList) const;
 	void GetModelParts(const lcMatrix44& WorldMatrix, int DefaultColorIndex, std::vector<lcModelPartsEntry>& ModelParts) const;
+	void CompareBoundingBox(const lcMatrix44& WorldMatrix, lcVector3& Min, lcVector3& Max) const;
+	void AddSubModelBoundingBoxPoints(const lcMatrix44& WorldMatrix, std::vector<lcVector3>& Points) const;
 	void UpdateBoundingBox(std::vector<lcModel*>& UpdatedModels);
 
 	void Load();
@@ -165,9 +183,8 @@ public:
 public:
 	char mFileName[LC_PIECE_NAME_LEN];
 	char m_strDescription[128];
-	int mZipFileType;
+	lcZipFileType mZipFileType;
 	int mZipFileIndex;
-	quint32 mFlags;
 	lcPieceInfoState mState;
 	int mFolderType;
 	int mFolderIndex;
@@ -176,6 +193,7 @@ protected:
 	void ReleaseMesh();
 
 	int mRefCount;
+	lcPieceInfoType mType;
 	lcModel* mModel;
 	Project* mProject;
 	lcMesh* mMesh;
