@@ -1,19 +1,12 @@
 #pragma once
 
-#include "object.h"
 #include "lc_array.h"
 #include "lc_application.h"
 
-#define LC_SCENE_BG				0x010	// Draw bg image
-#define LC_SCENE_BG_TILE		0x040	// Tile bg image
-#define LC_SCENE_GRADIENT		0x100	// Draw gradient
-
 #define LC_HTML_SINGLEPAGE    0x01
 #define LC_HTML_INDEX         0x02
-#define LC_HTML_IMAGES        0x04
 #define LC_HTML_LISTEND       0x08
 #define LC_HTML_LISTSTEP      0x10
-#define LC_HTML_HIGHLIGHT     0x20
 #define LC_HTML_SUBMODELS     0x40
 #define LC_HTML_CURRENT_ONLY  0x80
 
@@ -31,34 +24,25 @@ public:
 	bool IndexPage;
 	int StepImagesWidth;
 	int StepImagesHeight;
-	bool HighlightNewParts;
 	bool PartsListStep;
 	bool PartsListEnd;
-	bool PartsListImages;
-	int PartImagesColor;
-	int PartImagesWidth;
-	int PartImagesHeight;
-};
-
-enum LC_MOUSE_TRACK
-{
-	LC_TRACK_NONE,
-	LC_TRACK_LEFT,
-	LC_TRACK_RIGHT
 };
 
 class Project
 {
 public:
-	Project();
+	Project(bool IsPreview = false);
 	~Project();
 
+	Project(const Project&) = delete;
+	Project& operator=(const Project&) = delete;
+
 	const lcArray<lcModel*>& GetModels() const
 	{
 		return mModels;
 	}
 
-	lcModel* GetModel(const QString& Name) const;
+	lcModel* GetModel(const QString& FileName) const;
 
 	lcModel* GetActiveModel() const
 	{
@@ -86,14 +70,16 @@ public:
 
 	QString GetImageFileName(bool AllowCurrentFolder) const;
 
+	lcInstructions* GetInstructions();
+
 	void SetActiveModel(int ModelIndex);
-	void SetActiveModel(const QString& ModelName);
+	void SetActiveModel(const QString& FileName);
 
 	lcModel* CreateNewModel(bool ShowModel);
 	QString GetNewModelName(QWidget* ParentWidget, const QString& DialogTitle, const QString& CurrentName, const QStringList& ExistingModels) const;
 	void ShowModelListDialog();
 
-	bool Load(const QString& FileName);
+	bool Load(const QString& FileName, bool ShowErrors);
 	bool Save(const QString& FileName);
 	bool Save(QTextStream& Stream);
 	void Merge(Project* Other);
@@ -102,36 +88,37 @@ public:
 
 	void SaveImage();
 	bool ExportModel(const QString& FileName, lcModel* Model) const;
-	void Export3DStudio(const QString& FileName);
+	bool Export3DStudio(const QString& FileName);
 	void ExportBrickLink();
-	void ExportCOLLADA(const QString& FileName);
+	bool ExportCOLLADA(const QString& FileName);
 	void ExportCSV();
 	void ExportHTML(const lcHTMLExportOptions& Options);
 	bool ExportPOVRay(const QString& FileName);
-	void ExportWavefront(const QString& FileName);
+	bool ExportWavefront(const QString& FileName);
 
 	void UpdatePieceInfo(PieceInfo* Info) const;
 
 protected:
 	QString GetExportFileName(const QString& FileName, const QString& DefaultExtension, const QString& DialogTitle, const QString& DialogFilter) const;
+
 	std::vector<lcModelPartsEntry> GetModelParts();
-	QImage CreatePartsListImage(lcModel* Model, lcStep Step);
-	void CreateHTMLPieceList(QTextStream& Stream, lcModel* Model, lcStep Step, bool Images);
 	void SetFileName(const QString& FileName);
 
+	bool mIsPreview;
 	bool mModified;
 	QString mFileName;
 	QFileSystemWatcher mFileWatcher;
 
 	lcArray<lcModel*> mModels;
 	lcModel* mActiveModel;
+	std::unique_ptr<lcInstructions> mInstructions;
 
 	Q_DECLARE_TR_FUNCTIONS(Project);
 };
 
 inline lcModel* lcGetActiveModel()
 {
-	Project* Project = lcGetActiveProject();
+	const Project* const Project = lcGetActiveProject();
 	return Project ? Project->GetActiveModel() : nullptr;
 }
 
