@@ -4,7 +4,6 @@ class TexFont
 {
 public:
 	TexFont();
-	~TexFont();
 
 	bool IsLoaded() const
 	{
@@ -16,8 +15,8 @@ public:
 		return mTexture;
 	}
 
-	bool Load(lcContext* Context);
-	void Release();
+	bool Initialize(lcContext* Context);
+	void Reset();
 
 	void PrintText(lcContext* Context, float Left, float Top, float Z, const char* Text) const;
 	void GetTriangles(const lcMatrix44& Transform, const char* Text, float* Buffer) const;
@@ -35,7 +34,6 @@ protected:
 	int mTextureWidth;
 	int mTextureHeight;
 	int mFontHeight;
-	int mRefCount;
 };
 
 extern TexFont gTexFont;
