@@ -11,6 +11,8 @@
 #include "pieceinf.h"
 #include "lc_library.h"
 #include "lc_qutils.h"
+#include "lc_viewwidget.h"
+#include "lc_previewwidget.h"
 
 // Draw an icon indicating opened/closing branches
 static QIcon drawIndicatorIcon(const QPalette &palette, QStyle *style)
@@ -209,13 +211,8 @@ lcQPropertiesTree::lcQPropertiesTree(QWidget *parent) :
 	labels.append(tr("Property"));
 	labels.append(tr("Value"));
 	setHeaderLabels(labels);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
 	header()->setSectionsMovable(false);
 	header()->setSectionResizeMode(QHeaderView::ResizeToContents);
-#else
-	header()->setMovable(false);
-	header()->setResizeMode(QHeaderView::ResizeToContents);
-#endif
 	header()->setVisible(false);
 	setAlternatingRowColors(true);
 	setRootIsDecorated(false);
@@ -316,15 +313,15 @@ void lcQPropertiesTree::Update(const lcArray<lcObject*>& Selection, lcObject* Fo
 	{
 		switch (Focus->GetType())
 		{
-		case LC_OBJECT_PIECE:
+		case lcObjectType::Piece:
 			Mode = LC_PROPERTY_WIDGET_PIECE;
 			break;
 
-		case LC_OBJECT_CAMERA:
+		case lcObjectType::Camera:
 			Mode = LC_PROPERTY_WIDGET_CAMERA;
 			break;
 
-		case LC_OBJECT_LIGHT:
+		case lcObjectType::Light:
 			Mode = LC_PROPERTY_WIDGET_LIGHT;
 			break;
 		}
@@ -335,7 +332,7 @@ void lcQPropertiesTree::Update(const lcArray<lcObject*>& Selection, lcObject* Fo
 		{
 			switch (Selection[ObjectIdx]->GetType())
 			{
-			case LC_OBJECT_PIECE:
+			case lcObjectType::Piece:
 				if (Mode == LC_PROPERTY_WIDGET_EMPTY)
 					Mode = LC_PROPERTY_WIDGET_PIECE;
 				else if (Mode != LC_PROPERTY_WIDGET_PIECE)
@@ -345,7 +342,7 @@ void lcQPropertiesTree::Update(const lcArray<lcObject*>& Selection, lcObject* Fo
 				}
 				break;
 
-			case LC_OBJECT_CAMERA:
+			case lcObjectType::Camera:
 				if (Mode != LC_PROPERTY_WIDGET_EMPTY)
 				{
 					Mode = LC_PROPERTY_WIDGET_MULTIPLE;
@@ -358,7 +355,7 @@ void lcQPropertiesTree::Update(const lcArray<lcObject*>& Selection, lcObject* Fo
 				}
 				break;
 
-			case LC_OBJECT_LIGHT:
+			case lcObjectType::Light:
 				if (Mode != LC_PROPERTY_WIDGET_EMPTY)
 				{
 					Mode = LC_PROPERTY_WIDGET_MULTIPLE;
@@ -398,6 +395,34 @@ void lcQPropertiesTree::Update(const lcArray<lcObject*>& Selection, lcObject* Fo
 	}
 }
 
+class lcStepValidator : public QIntValidator
+{
+public:
+	lcStepValidator(lcStep Min, lcStep Max, bool AllowEmpty)
+		: QIntValidator(1, INT_MAX), mMin(Min), mMax(Max), mAllowEmpty(AllowEmpty)
+	{
+	}
+
+	QValidator::State validate(QString& Input, int& Pos) const override
+	{
+		if (mAllowEmpty && Input.isEmpty())
+			return Acceptable;
+
+		bool Ok;
+		lcStep Step = Input.toUInt(&Ok);
+
+		if (Ok)
+			return (Step >= mMin && Step <= mMax) ? Acceptable : Invalid;
+
+		return QIntValidator::validate(Input, Pos);
+	}
+
+protected:
+	lcStep mMin;
+	lcStep mMax;
+	bool mAllowEmpty;
+};
+
 QWidget *lcQPropertiesTree::createEditor(QWidget *parent, QTreeWidgetItem *item) const
 {
 	PropertyType propertyType = (PropertyType)item->data(0, lcQPropertiesTree::PropertyTypeRole).toInt();
@@ -432,23 +457,36 @@ QWidget *lcQPropertiesTree::createEditor(QWidget *parent, QTreeWidgetItem *item)
 			return editor;
 		}
 
-	case PropertyInt:
+	case PropertyStep:
 		{
-			QLineEdit *editor = new QLineEdit(parent);
-			quint32 value = item->data(0, PropertyValueRole).toUInt();
+			QLineEdit* Editor = new QLineEdit(parent);
 
-			editor->setValidator(new QIntValidator());
-			editor->setText(QString::number(value));
+			lcStep Value = item->data(0, PropertyValueRole).toUInt();
+			lcStep Show = partShow->data(0, PropertyValueRole).toUInt();
+			lcStep Hide = partHide->data(0, PropertyValueRole).toUInt();
 
-			connect(editor, SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
+			if (Show && Hide)
+			{
+				if (item == partShow)
+					Editor->setValidator(new lcStepValidator(1, Hide - 1, false));
+				else
+					Editor->setValidator(new lcStepValidator(Show + 1, LC_STEP_MAX, true));
+			}
+			else
+				Editor->setValidator(new lcStepValidator(1, LC_STEP_MAX, item == partHide));
 
-			return editor;
+			if (item != partHide || Value != LC_STEP_MAX)
+				Editor->setText(QString::number(Value));
+
+			connect(Editor, SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
+
+			return Editor;
 		}
 
 	case PropertyString:
 		{
 			QLineEdit *editor = new QLineEdit(parent);
-			const char *value = (const char*)item->data(0, PropertyValueRole).value<void*>();
+			QString value = item->data(0, PropertyValueRole).toString();
 
 			editor->setText(value);
 
@@ -491,10 +529,10 @@ QWidget *lcQPropertiesTree::createEditor(QWidget *parent, QTreeWidgetItem *item)
 			std::sort(SortedPieces.begin(), SortedPieces.end(), PieceCompare);
 
 			for (PieceInfo* Info : SortedPieces)
-				editor->addItem(Info->m_strDescription, qVariantFromValue((void*)Info));
+				editor->addItem(Info->m_strDescription, QVariant::fromValue((void*)Info));
 
 			PieceInfo *info = (PieceInfo*)item->data(0, PropertyValueRole).value<void*>();
-			editor->setCurrentIndex(editor->findData(qVariantFromValue((void*)info)));
+			editor->setCurrentIndex(editor->findData(QVariant::fromValue((void*)info)));
 
 			connect(editor, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSetValue(int)));
 
@@ -595,15 +633,26 @@ void lcQPropertiesTree::slotReturnPressed()
 		}
 		else if (Item == partShow)
 		{
-			lcStep Step = Editor->text().toUInt();
+			bool Ok = false;
+			lcStep Step = Editor->text().toUInt(&Ok);
 
-			Model->SetSelectedPiecesStepShow(Step);
+			if (Ok)
+				Model->SetSelectedPiecesStepShow(Step);
 		}
 		else if (Item == partHide)
 		{
-			lcStep Step = Editor->text().toUInt();
+			QString Text = Editor->text();
 
-			Model->SetSelectedPiecesStepHide(Step);
+			if (Text.isEmpty())
+				Model->SetSelectedPiecesStepHide(LC_STEP_MAX);
+			else
+			{
+				bool Ok = false;
+				lcStep Step = Text.toUInt(&Ok);
+
+				if (Ok)
+					Model->SetSelectedPiecesStepHide(Step);
+			}
 		}
 	}
 	else if (mWidgetMode == LC_PROPERTY_WIDGET_CAMERA)
@@ -685,7 +734,7 @@ void lcQPropertiesTree::slotReturnPressed()
 			{
 				QString Value = Editor->text();
 
-				Model->SetCameraName(Camera, Value.toLocal8Bit().data());
+				Model->SetCameraName(Camera, Value);
 			}
 		}
 	}
@@ -709,7 +758,15 @@ void lcQPropertiesTree::slotSetValue(int Value)
 		{
 			QComboBox *editor = (QComboBox*)sender();
 
-			Model->SetSelectedPiecesPieceInfo((PieceInfo*)editor->itemData(Value).value<void*>());
+			PieceInfo* Info = (PieceInfo*)editor->itemData(Value).value<void*>();
+			Model->SetSelectedPiecesPieceInfo(Info);
+
+			int ColorIndex = gDefaultColor;
+			lcObject* Focus = gMainWindow->GetActiveModel()->GetFocusObject();
+			if (Focus && Focus->IsPiece())
+				ColorIndex = ((lcPiece*)Focus)->GetColorIndex();
+			quint32 ColorCode = lcGetColorCode(ColorIndex);
+			gMainWindow->PreviewPiece(Info->mFileName, ColorCode, false);
 		}
 	}
 }
@@ -718,30 +775,42 @@ void lcQPropertiesTree::slotColorButtonClicked()
 {
 	int ColorIndex = gDefaultColor;
 	lcObject* Focus = gMainWindow->GetActiveModel()->GetFocusObject();
+
 	if (Focus && Focus->IsPiece())
-		ColorIndex = ((lcPiece*)Focus)->mColorIndex;
+		ColorIndex = ((lcPiece*)Focus)->GetColorIndex();
 
-	QWidget *parent = (QWidget*)sender();
-	lcQColorPickerPopup *popup = new lcQColorPickerPopup(parent, ColorIndex);
-	connect(popup, SIGNAL(selected(int)), SLOT(slotSetValue(int)));
-	popup->setMinimumSize(300, 200);
+	QWidget* Button = (QWidget*)sender();
 
-	const QRect desktop = QApplication::desktop()->geometry();
+	if (!Button)
+		return;
 
-	QPoint pos = parent->mapToGlobal(parent->rect().bottomLeft());
-	if (pos.x() < desktop.left())
-		pos.setX(desktop.left());
-	if (pos.y() < desktop.top())
-		pos.setY(desktop.top());
+	lcQColorPickerPopup* Popup = new lcQColorPickerPopup(Button, ColorIndex);
+	connect(Popup, SIGNAL(selected(int)), SLOT(slotSetValue(int)));
+	Popup->setMinimumSize(qMax(300, width()), qMax(200, static_cast<int>(width() * 2 / 3)));
 
-	if ((pos.x() + popup->width()) > desktop.width())
-		pos.setX(desktop.width() - popup->width());
-	if ((pos.y() + popup->height()) > desktop.bottom())
-		pos.setY(desktop.bottom() - popup->height());
-	popup->move(pos);
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
+	QScreen* Screen = QGuiApplication::screenAt(Button->mapToGlobal(Button->rect().bottomLeft()));
+	const QRect ScreenRect = Screen ? Screen->geometry() : QApplication::desktop()->geometry();
+#else
+	const QRect ScreenRect = QApplication::desktop()->geometry();
+#endif
 
-	popup->setFocus();
-	popup->show();
+	int x = mapToGlobal(QPoint(0, 0)).x();
+	int y = Button->mapToGlobal(Button->rect().bottomLeft()).y();
+
+	if (x < ScreenRect.left())
+		x = ScreenRect.left();
+	if (y < ScreenRect.top())
+		y = ScreenRect.top();
+
+	if (x + Popup->width() > ScreenRect.right())
+		x = ScreenRect.right() - Popup->width();
+	if (y + Popup->height() > ScreenRect.bottom())
+		y = ScreenRect.bottom() - Popup->height();
+
+	Popup->move(QPoint(x, y));
+	Popup->setFocus();
+	Popup->show();
 }
 
 QTreeWidgetItem *lcQPropertiesTree::addProperty(QTreeWidgetItem *parent, const QString& label, PropertyType propertyType)
@@ -755,7 +824,7 @@ QTreeWidgetItem *lcQPropertiesTree::addProperty(QTreeWidgetItem *parent, const Q
 
 	newItem->setData(0, PropertyTypeRole, QVariant(propertyType));
 	newItem->setFlags(newItem->flags() | Qt::ItemIsEditable);
-	setItemExpanded(newItem, true);
+	newItem->setExpanded(true);
 
 	if (propertyType == PropertyGroup)
 	{
@@ -824,9 +893,9 @@ void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject*
 		partRotationY = addProperty(partRotation, tr("Y"), PropertyFloat);
 		partRotationZ = addProperty(partRotation, tr("Z"), PropertyFloat);
 
-		partVisibility = addProperty(nullptr, tr("Visibility"), PropertyGroup);
-		partShow = addProperty(partVisibility, tr("Show"), PropertyInt);
-		partHide = addProperty(partVisibility, tr("Hide"), PropertyInt);
+		partVisibility = addProperty(nullptr, tr("Visible Steps"), PropertyGroup);
+		partShow = addProperty(partVisibility, tr("Show"), PropertyStep);
+		partHide = addProperty(partVisibility, tr("Hide"), PropertyStep);
 
 		partAppearance = addProperty(nullptr, tr("Appearance"), PropertyGroup);
 		partColor = addProperty(partAppearance, tr("Color"), PropertyColor);
@@ -870,8 +939,10 @@ void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject*
 	{
 		Show = Piece->GetStepShow();
 		Hide = Piece->GetStepHide();
-		ColorIndex = Piece->mColorIndex;
+		ColorIndex = Piece->GetColorIndex();
 		Info = Piece->mPieceInfo;
+		quint32 ColorCode = lcGetColorCode(ColorIndex);
+		gMainWindow->PreviewPiece(Info->mFileName, ColorCode, false);
 	}
 	else
 	{
@@ -881,7 +952,7 @@ void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject*
 		{
 			lcObject* Object = Selection[ObjectIdx];
 
-			if (Object->GetType() != LC_OBJECT_PIECE)
+			if (!Object->IsPiece())
 				continue;
 
 			lcPiece* SelectedPiece = (lcPiece*)Object;
@@ -890,7 +961,7 @@ void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject*
 			{
 				Show = SelectedPiece->GetStepShow();
 				Hide = SelectedPiece->GetStepHide();
-				ColorIndex = SelectedPiece->mColorIndex;
+				ColorIndex = SelectedPiece->GetColorIndex();
 				Info = SelectedPiece->mPieceInfo;
 
 				FirstPiece = false;
@@ -903,7 +974,7 @@ void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject*
 				if (SelectedPiece->GetStepHide() != Hide)
 					Hide = 0;
 
-				if (SelectedPiece->mColorIndex != ColorIndex)
+				if (SelectedPiece->GetColorIndex() != ColorIndex)
 					ColorIndex = gDefaultColor;
 
 				if (SelectedPiece->mPieceInfo != Info)
@@ -914,7 +985,7 @@ void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject*
 
 	partShow->setText(1, QString::number(Show));
 	partShow->setData(0, PropertyValueRole, Show);
-	partHide->setText(1, QString::number(Hide));
+	partHide->setText(1, Hide == LC_STEP_MAX ? QString() : QString::number(Hide));
 	partHide->setData(0, PropertyValueRole, Hide);
 
 	QImage img(16, 16, QImage::Format_ARGB32);
@@ -935,7 +1006,7 @@ void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject*
 	QString text = Info ? Info->m_strDescription : QString();
 	partID->setText(1, text);
 	partID->setToolTip(1, text);
-	partID->setData(0, PropertyValueRole, qVariantFromValue((void*)Info));
+	partID->setData(0, PropertyValueRole, QVariant::fromValue((void*)Info));
 }
 
 void lcQPropertiesTree::SetCamera(lcObject* Focus)
@@ -979,7 +1050,7 @@ void lcQPropertiesTree::SetCamera(lcObject* Focus)
 	float FoV = 60.0f;
 	float ZNear = 1.0f;
 	float ZFar = 100.0f;
-	const char* Name = "";
+	QString Name;
 
 	if (Camera)
 	{
@@ -1025,7 +1096,7 @@ void lcQPropertiesTree::SetCamera(lcObject* Focus)
 	cameraFar->setData(0, PropertyValueRole, ZFar);
 
 	cameraName->setText(1, Name);
-	cameraName->setData(0, PropertyValueRole, qVariantFromValue((void*)Name));
+	cameraName->setData(0, PropertyValueRole, Name);
 }
 
 void lcQPropertiesTree::SetLight(lcObject* Focus)
