@@ -60,36 +60,27 @@ bool lcQTreeWidgetColumnStretcher::eventFilter(QObject *obj, QEvent *ev)
 	{
 		if (ev->type() == QEvent::Show)
 		{
-			QHeaderView *hv = qobject_cast<QHeaderView*>(obj);
-			for (int i = 0; i < hv->count(); ++i)
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
-				hv->setSectionResizeMode(i, QHeaderView::Interactive);
-#else
-				hv->setResizeMode(i, QHeaderView::Interactive);
-#endif
+			QHeaderView* HeaderView = qobject_cast<QHeaderView*>(obj);
+
+			for (int i = 0; i < HeaderView->count(); ++i)
+				HeaderView->setSectionResizeMode(i, QHeaderView::Interactive);
 		}
 		else if (ev->type() == QEvent::Hide)
 		{
-			QHeaderView *hv = qobject_cast<QHeaderView*>(obj);
-			for (int i = 0; i < hv->count(); ++i)
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
-				hv->setSectionResizeMode(i, i == m_columnToStretch ? QHeaderView::Stretch : QHeaderView::ResizeToContents);
-#else
-				hv->setResizeMode(i, i == m_columnToStretch ? QHeaderView::Stretch : QHeaderView::ResizeToContents);
-#endif
+			QHeaderView* HeaderView = qobject_cast<QHeaderView*>(obj);
+
+			for (int i = 0; i < HeaderView->count(); ++i)
+				HeaderView->setSectionResizeMode(i, i == m_columnToStretch ? QHeaderView::Stretch : QHeaderView::ResizeToContents);
 		}
 		else if (ev->type() == QEvent::Resize)
 		{
-			QHeaderView *hv = qobject_cast<QHeaderView*>(obj);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
-			if (hv->sectionResizeMode(m_columnToStretch) == QHeaderView::Interactive)
-#else
-			if (hv->resizeMode(m_columnToStretch) == QHeaderView::Interactive)
-#endif
+			QHeaderView* HeaderView = qobject_cast<QHeaderView*>(obj);
+
+			if (HeaderView->sectionResizeMode(m_columnToStretch) == QHeaderView::Interactive)
 			{
 				QResizeEvent *re = static_cast<QResizeEvent*>(ev);
 				int diff = re->size().width() - re->oldSize().width() ;
-				hv->resizeSection(m_columnToStretch, qMax(32, hv->sectionSize(1) + diff));
+				HeaderView->resizeSection(m_columnToStretch, qMax(32, HeaderView->sectionSize(1) + diff));
 			}
 		}
 	}
