@@ -8,33 +8,41 @@ float lcParseValueLocalized(const QString& Value);
 
 class lcQTreeWidgetColumnStretcher : public QObject
 {
-public:
-	lcQTreeWidgetColumnStretcher(QTreeWidget *treeWidget, int columnToStretch);
+	Q_OBJECT
 
-	virtual bool eventFilter(QObject *obj, QEvent *ev);
+public:
+	lcQTreeWidgetColumnStretcher(QTreeWidget* TreeWidget, int ColumnToStretch);
+
+	bool eventFilter(QObject* Object, QEvent* Event) override;
 
 	const int m_columnToStretch;
 };
 
-class lcTransformLineEdit : public QLineEdit
+class lcSmallLineEdit : public QLineEdit
 {
-public:
-	lcTransformLineEdit()
-		: QLineEdit()
-	{
-	}
+	Q_OBJECT
 
-	virtual QSize sizeHint() const override
+public:
+	QSize sizeHint() const override
 	{
 		QFontMetrics FontMetrics(font());
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 11, 0))
+		int Width = FontMetrics.horizontalAdvance(QLatin1Char('x')) * 10;
+#else
 		int Width = FontMetrics.width(QLatin1Char('x')) * 10;
+#endif
 
 		return QLineEdit::sizeHint() - QSize(Width, 0);
 	}
+};
+
+class lcTransformLineEdit : public lcSmallLineEdit
+{
+	Q_OBJECT
 
 protected:
-	virtual bool event(QEvent* Event) override
+	bool event(QEvent* Event) override
 	{
 		if (Event->type() == QEvent::ShortcutOverride)
 		{
