@@ -155,7 +155,6 @@ void lcSetsDatabaseDialog::on_SearchButton_clicked()
 
 void lcSetsDatabaseDialog::DownloadFinished(lcHttpReply* Reply)
 {
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
 	if (Reply == mKeyListReply)
 	{
 		if (!Reply->error())
@@ -227,7 +226,6 @@ void lcSetsDatabaseDialog::DownloadFinished(lcHttpReply* Reply)
 
 		mInventoryReply = nullptr;
 	}
-#endif
 
 	Reply->deleteLater();
 }
