@@ -1,7 +1,7 @@
 #include "lc_global.h"
 #include "lc_application.h"
 #include "lc_qupdatedialog.h"
-#include "lc_mainwindow.h"
+#include "lc_profile.h"
 #include <QApplication>
 #include <locale.h>
 
@@ -70,7 +70,7 @@ static LONG WINAPI lcSehHandler(PEXCEPTION_POINTERS exceptionPointers)
 
 static void lcSehInit()
 {
-	if (GetTempPath(sizeof(gMinidumpPath) / sizeof(gMinidumpPath[0]), gMinidumpPath))
+	if (GetTempPath(LC_ARRAY_COUNT(gMinidumpPath), gMinidumpPath))
 		lstrcat(gMinidumpPath, TEXT("leocad.dmp"));
 
 	SetUnhandledExceptionFilter(lcSehHandler);
@@ -121,28 +121,56 @@ static void lcRegisterShellFileTypes()
 
 #endif
 
+static void lcInitializeSurfaceFormat(int argc, char* argv[])
+{
+	QCoreApplication Application(argc, argv);
+	const lcCommandLineOptions Options = lcApplication::ParseCommandLineOptions();
+
+	if (Options.ParseOK && Options.AASamples > 1)
+	{
+		QSurfaceFormat Format = QSurfaceFormat::defaultFormat();
+		Format.setSamples(Options.AASamples);
+		QSurfaceFormat::setDefaultFormat(Format);
+	}
+}
+
 int main(int argc, char *argv[])
 {
+	QCoreApplication::setOrganizationDomain(QLatin1String("leocad.org"));
+	QCoreApplication::setOrganizationName(QLatin1String("LeoCAD Software"));
+	QCoreApplication::setApplicationName(QLatin1String("LeoCAD"));
+	QCoreApplication::setApplicationVersion(QLatin1String(LC_VERSION_TEXT));
+
+	lcInitializeSurfaceFormat(argc, argv);
+
+	QCoreApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
+
 	lcApplication Application(argc, argv);
 
+	QString Language = lcGetProfileString(LC_PROFILE_LANGUAGE);
+	QLocale Locale;
+
+	if (!Language.isEmpty())
+		Locale = QLocale(Language);
+
 	QTranslator QtTranslator;
-	if (QtTranslator.load(QLocale::system(), "qt", "_", QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
+	if (QtTranslator.load(Locale, "qt", "_", QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
 		Application.installTranslator(&QtTranslator);
 #ifdef Q_OS_WIN
-	else if (QtTranslator.load(QLocale::system(), "qt", "_", qApp->applicationDirPath() + "/translations"))
+	else if (QtTranslator.load(Locale, "qt", "_", qApp->applicationDirPath() + "/translations"))
 		Application.installTranslator(&QtTranslator);
 #endif
 
 	QTranslator QtBaseTranslator;
-	if (QtBaseTranslator.load("qtbase_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
+	if (QtBaseTranslator.load("qtbase_" + Locale.name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
 		Application.installTranslator(&QtBaseTranslator);
 #ifdef Q_OS_WIN
-	else if (QtBaseTranslator.load("qtbase_" + QLocale::system().name(), qApp->applicationDirPath() + "/translations"))
+	else if (QtBaseTranslator.load("qtbase_" + Locale.name(), qApp->applicationDirPath() + "/translations"))
 		Application.installTranslator(&QtBaseTranslator);
 #endif
 
 	QTranslator Translator;
-	if (Translator.load("leocad_" + QLocale::system().name(), ":/resources"))
+	if (Translator.load("leocad_" + Locale.name(), ":/resources"))
 		Application.installTranslator(&Translator);
 
 	qRegisterMetaTypeStreamOperators<QList<int> >("QList<int>");
@@ -170,13 +198,17 @@ int main(int argc, char *argv[])
 	
 	setlocale(LC_NUMERIC, "C");
 
-	bool ShowWindow;
-	if (!Application.Initialize(LibraryPaths, ShowWindow))
+	lcStartupMode StartupMode = Application.Initialize(LibraryPaths);
+
+	if (StartupMode == lcStartupMode::Error)
+	{
+		Application.Shutdown();
 		return 1;
+	}
 
 	int ExecReturn = 0;
 
-	if (ShowWindow)
+	if (StartupMode == lcStartupMode::ShowWindow)
 	{
 #if !LC_DISABLE_UPDATE_CHECK
 		lcDoInitialUpdateCheck();
@@ -185,8 +217,7 @@ int main(int argc, char *argv[])
 		ExecReturn = Application.exec();
 	}
 
-	delete gMainWindow;
-	gMainWindow = nullptr;
+	Application.Shutdown();
 
 	return ExecReturn;
 }
