@@ -1,6 +1,6 @@
 #include "lc_global.h"
 
-#ifdef WIN32
+#ifdef Q_OS_WIN
 
 char* strcasestr(const char *s, const char *find)
 {
@@ -33,12 +33,4 @@ char* strupr(char *string)
 	return string;
 }
 
-char* strlwr(char *string)
-{
-	for (char *c = string; *c; c++)
-		*c = tolower(*c);
-
-	return string;
-}
-
 #endif
